/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import { PresetColor } from "./Types";
import { Control } from "./Control";
import { View } from "./View";
import { ImageView } from "./ImageView";
import { Label } from "./Label";
import { ControlState } from "./Enum";
/**
 * Button
 */
export class Button extends Control {
    textColor() {
        return this.props.color || this.props.tintColor;
    }
    content() {
        var imageStyle = {
            flex: 0,
            alignSelf: "center",
            marginRight: this.props.middleInset / 2.0,
        };
        var labelStyle = {
            flex: 0,
            alignSelf: 'center',
            marginLeft: this.props.middleInset / 2.0,
        };
        if (!!this.props.image) {
            return (React.createElement(View, {style: { flexDirection: 'row' }}, React.createElement(ImageView, {image: this.props.image, tintColor: this.textColor(), simpleImage: true, style: imageStyle}), React.createElement(Label, __assign({style: labelStyle, textColor: this.textColor(), text: this.props.title}, this.props.titleLabelProps))));
        }
        else {
            return (React.createElement(View, {style: { flexDirection: 'row' }}, React.createElement(Label, __assign({style: labelStyle, textColor: this.textColor(), text: this.props.title}, this.props.titleLabelProps))));
        }
    }
}
Button.defaultProps = Object.assign({}, Control.defaultProps, {
    title: "",
    titleLabelProps: {},
    middleInset: 0.0,
});
/**
 * TextButton
 */
export class TextButton extends Button {
    textColor() {
        if (typeof this.props.color === "string") {
            return this.props.color;
        }
        else if (this.props.lighted) {
            if (this.controlState() == ControlState.Disabled) {
                return "#4D4D4D";
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return this.props.tintColor;
            }
            return "#FFFFFF";
        }
        else if (this.props.reversed) {
            if (this.controlState() == ControlState.Disabled) {
                return PresetColor.bgColor;
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return this.props.tintColor;
            }
            return PresetColor.bgColor;
        }
        else {
            if (this.controlState() == ControlState.Disabled) {
                return this.props.tintColorDisabled;
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return PresetColor.bgColor;
            }
            return this.props.tintColor;
        }
    }
}
/**
 * RectangleButton
 */
export class RectangleButton extends Button {
    textColor() {
        if (typeof this.props.color === "string") {
            return this.props.color;
        }
        else if (this.props.lighted) {
            if (this.controlState() == ControlState.Disabled) {
                return "#4D4D4D";
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return this.props.tintColor;
            }
            else if (this.controlState() == ControlState.Selected) {
                return this.props.tintColor;
            }
            return "#FFFFFF";
        }
        else if (this.props.reversed) {
            if (this.controlState() == ControlState.Disabled) {
                return PresetColor.bgColor;
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return this.props.tintColor;
            }
            else if (this.controlState() == ControlState.Selected) {
                return this.props.tintColor;
            }
            return PresetColor.bgColor;
        }
        else {
            if (this.controlState() == ControlState.Disabled) {
                return this.props.tintColorDisabled;
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return PresetColor.bgColor;
            }
            else if (this.controlState() == ControlState.Selected) {
                return PresetColor.bgColor;
            }
            return this.props.tintColor;
        }
    }
    customStyle() {
        if (this.props.lighted) {
            if (this.controlState() == ControlState.Disabled) {
                return {
                    borderColor: "#4D4D4D",
                    backgroundColor: "#FFFFFF00"
                };
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return {
                    borderColor: "#FFFFFF",
                    backgroundColor: this.props.highlightedColor ? this.props.highlightedColor : "#FFFFFF"
                };
            }
            else if (this.controlState() == ControlState.Selected) {
                return {
                    borderColor: "#FFFFFF",
                    backgroundColor: "#FFFFFF"
                };
            }
            return {
                borderColor: "#FFFFFF",
                backgroundColor: "#FFFFFF00"
            };
        }
        else if (this.props.reversed) {
            if (this.controlState() == ControlState.Disabled) {
                return {
                    borderColor: this.props.tintColorDisabled,
                    backgroundColor: this.props.tintColorDisabled,
                };
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return {
                    borderColor: this.props.tintColor,
                    backgroundColor: this.props.highlightedColor ? this.props.highlightedColor : PresetColor.bgColor
                };
            }
            else if (this.controlState() == ControlState.Selected) {
                return {
                    borderColor: this.props.tintColor,
                    backgroundColor: PresetColor.bgColor
                };
            }
            return {
                borderColor: this.props.tintColor,
                backgroundColor: this.props.tintColor
            };
        }
        else {
            if (this.controlState() == ControlState.Disabled) {
                return {
                    borderColor: this.props.tintColorDisabled,
                };
            }
            else if (this.controlState() == ControlState.Highlighted) {
                return {
                    borderColor: this.props.tintColor,
                    backgroundColor: this.props.highlightedColor ? this.props.highlightedColor : this.props.tintColor
                };
            }
            else if (this.controlState() == ControlState.Selected) {
                return {
                    borderColor: this.props.tintColor,
                    backgroundColor: this.props.tintColor
                };
            }
            return {
                borderColor: this.props.tintColor,
                backgroundColor: PresetColor.bgColor
            };
        }
    }
}
RectangleButton.defaultProps = Object.assign({}, Button.defaultProps, {
    cornerRadius: 6.0,
    borderWidth: 1.0,
    highlightedColor: null,
});
