import * as RN from "react-native";
import { GestureRecognizerState, SwipeGestureRecognizerDirection } from "./Enum";
export class GestureRecognizer {
    constructor(handler) {
        this.state = GestureRecognizerState.Possible;
        this.enabled = true;
        this.values = {
            touches: {},
            taps: 0,
            last: undefined,
            touchIDs: [],
        };
        this.handler = handler;
    }
    touchesBegan(view, touchEvent, args) { }
    touchesMoved(view, touchEvent, args) { }
    touchesEnded(view, touchEvent, args) { }
    touchesCancelled(view, touchEvent, args) { }
    static lastTouch(values) {
        let fingerID = values.touchIDs[values.touchIDs.length - 1];
        if (typeof fingerID === "string" || typeof fingerID === "number") {
            let lastTouch = values.touches[fingerID];
            return lastTouch;
        }
        return undefined;
    }
    updateValues(event) {
        var nativeEvent = event.nativeEvent;
        if (!!!nativeEvent) {
            return;
        }
        if (RN.Platform.OS == "android") {
            if (this.values.touchIDs.length === 0) {
                this.values.touches[nativeEvent.identifier] = {
                    _startX: nativeEvent.pageX,
                    _startY: nativeEvent.pageY,
                    startX: nativeEvent.locationX,
                    startY: nativeEvent.locationY,
                    currentX: nativeEvent.locationX + nativeEvent.pageX - nativeEvent.locationX,
                    currentY: nativeEvent.locationY + nativeEvent.pageY - nativeEvent.locationY,
                    windowX: nativeEvent.pageX,
                    windowY: nativeEvent.pageY,
                    offsetX: 0,
                    offsetY: 0,
                };
                if (this.values.touchIDs.indexOf(nativeEvent.identifier) < 0) {
                    this.values.touchIDs.push(nativeEvent.identifier);
                    this.values.taps = this.values.touchIDs.length;
                }
            }
            else {
                if (this.values.touches[nativeEvent.identifier]) {
                    this.values.touches[nativeEvent.identifier].currentX = nativeEvent.locationX + nativeEvent.pageX - this.values.touches[nativeEvent.identifier]._startX;
                    this.values.touches[nativeEvent.identifier].currentX = nativeEvent.locationY + nativeEvent.pageY - this.values.touches[nativeEvent.identifier]._startY;
                    this.values.touches[nativeEvent.identifier].windowX = nativeEvent.pageX;
                    this.values.touches[nativeEvent.identifier].windowY = nativeEvent.pageY;
                    this.values.touches[nativeEvent.identifier].offsetX = nativeEvent.pageX - this.values.touches[nativeEvent.identifier]._startX;
                    this.values.touches[nativeEvent.identifier].offsetY = nativeEvent.pageY - this.values.touches[nativeEvent.identifier]._startY;
                }
            }
        }
        else {
            if (this.values.touchIDs.length === 0) {
                this.values.touches[nativeEvent.identifier] = {
                    startX: nativeEvent.locationX,
                    startY: nativeEvent.locationY,
                    currentX: nativeEvent.locationX,
                    currentY: nativeEvent.locationY,
                    windowX: nativeEvent.pageX,
                    windowY: nativeEvent.pageY,
                    offsetX: 0,
                    offsetY: 0,
                };
                if (this.values.touchIDs.indexOf(nativeEvent.identifier) < 0) {
                    this.values.touchIDs.push(nativeEvent.identifier);
                    this.values.taps = this.values.touchIDs.length;
                }
            }
            else {
                if (this.values.touches[nativeEvent.identifier]) {
                    this.values.touches[nativeEvent.identifier].currentX = nativeEvent.locationX;
                    this.values.touches[nativeEvent.identifier].currentX = nativeEvent.locationY;
                    this.values.touches[nativeEvent.identifier].windowX = nativeEvent.pageX;
                    this.values.touches[nativeEvent.identifier].windowY = nativeEvent.pageY;
                    this.values.touches[nativeEvent.identifier].offsetX = nativeEvent.locationX - this.values.touches[nativeEvent.identifier].startX;
                    this.values.touches[nativeEvent.identifier].offsetY = nativeEvent.locationY - this.values.touches[nativeEvent.identifier].startY;
                }
            }
        }
    }
    resetValues() {
        this.values = {
            touches: {},
            taps: 0,
            last: new Date(),
            touchIDs: [],
        };
    }
}
export class TapGestureRecognizer extends GestureRecognizer {
    constructor(...args) {
        super(...args);
        this.maxIntervalOfTapsRequired = 350;
        this.numberOfTapsRequired = 1;
        this.numberOfTouchesRequired = 1;
    }
    touchesBegan(view, touchEvent, args) {
        if (this.numberOfTapsRequired > 1) {
            var current = new Date().getTime();
            var last = this.values.last.getTime();
            if (!(current - last < this.maxIntervalOfTapsRequired && this.validLastTouch(touchEvent))) {
                this.resetValues();
            }
            this.values.taps++;
            this.updateValues(touchEvent);
            if (this.values.taps >= this.numberOfTapsRequired) {
                this.handler(GestureRecognizerState.Began, this.values);
            }
        }
        else {
            this.resetValues();
            this.updateValues(touchEvent);
            this.handler(GestureRecognizerState.Began, this.values);
        }
    }
    touchesMoved(view, touchEvent, args) {
        this.updateValues(touchEvent);
        this.handler(GestureRecognizerState.Changed, this.values);
    }
    touchesEnded(view, touchEvent, args) {
        this.updateValues(touchEvent);
        var touches = () => {
            if (this.numberOfTouchesRequired > 1) {
                return this.values.touches[this.numberOfTouchesRequired - 1] !== undefined &&
                    parseInt(touchEvent.nativeEvent.identifier) === this.numberOfTouchesRequired - 1 &&
                    this.validTouch(view);
            }
            else {
                return this.validTouch(view);
            }
        };
        var taps = () => {
            if (this.numberOfTapsRequired > 1) {
                return this.values.taps >= this.numberOfTapsRequired;
            }
            else {
                return true;
            }
        };
        if (touches() && taps()) {
            this.handler(GestureRecognizerState.Ended, this.values);
            if (this.numberOfTapsRequired > 1 && this.values.taps >= this.numberOfTapsRequired) {
                this.resetValues();
            }
        }
        else {
            this.handler(GestureRecognizerState.Cancelled, this.values);
            this.values.last = new Date();
        }
    }
    touchesCancelled(view, touchEvent, args) {
        this.updateValues(touchEvent);
        this.handler(GestureRecognizerState.Cancelled, this.values);
        this.values.last = new Date();
    }
    validLastTouch(event) {
        if (this.numberOfTouchesRequired > 1) {
            return true;
        }
        else {
            let fingerID = this.values.touchIDs[this.values.touchIDs.length - 1];
            if (typeof fingerID === "string" || typeof fingerID === "number") {
                let lastTouch = this.values.touches[fingerID];
                let nextTouch = event.nativeEvent;
                if (lastTouch === undefined || nextTouch === undefined) {
                    return false;
                }
                var offsetX = nextTouch.locationX - lastTouch.startX;
                var offsetY = nextTouch.locationY - lastTouch.startY;
                return offsetX > -22.0 && offsetX < 22.0 &&
                    offsetY > -22.0 && offsetY < 22.0;
            }
            else {
                return false;
            }
        }
    }
    validTouch(view) {
        if (this.numberOfTouchesRequired > 1) {
            for (var key in this.values.touches) {
                if (this.values.touches.hasOwnProperty(key)) {
                    var touch = this.values.touches[key];
                    if ((touch.startX + touch.offsetX < view.bounds.width &&
                        touch.startY + touch.offsetY < view.bounds.height &&
                        touch.offsetX > -22.0 && touch.offsetX < 22.0 &&
                        touch.offsetY > -22.0 && touch.offsetY < 22.0) === false) {
                        return false;
                    }
                }
            }
            return true;
        }
        else {
            let fingerID = this.values.touchIDs[this.values.touchIDs.length - 1];
            if (typeof fingerID === "string" || typeof fingerID === "number") {
                let touch = this.values.touches[fingerID];
                if (touch === undefined) {
                    return false;
                }
                return touch.startX + touch.offsetX < view.bounds.width &&
                    touch.startY + touch.offsetY < view.bounds.height &&
                    touch.offsetX > -22.0 && touch.offsetX < 22.0 &&
                    touch.offsetY > -22.0 && touch.offsetY < 22.0;
            }
            else {
                return false;
            }
        }
    }
}
export class LongPressGestureRecognizer extends GestureRecognizer {
    constructor(...args) {
        super(...args);
        this.minimumPressDuration = 350;
        this.allowableMovement = 10;
        this.numberOfTouchesRequired = 1;
        this.recognizered = false;
        this.canceled = false;
        this.start = {};
        this.startDate = new Date();
        this.timer = undefined;
        this.fingerID = 0;
    }
    touchesBegan(view, touchEvent, args) {
        this.resetValues();
        this.updateValues(touchEvent);
        this.timer = setTimeout(() => {
            if (this.canceled) {
                return;
            }
            if (this.validTouch(view)) {
                this.recognizered = true;
                this.handler(GestureRecognizerState.Began, this.values);
            }
        }, this.minimumPressDuration);
    }
    touchesMoved(view, touchEvent, args) {
        if (this.canceled) {
            return;
        }
        this.updateValues(touchEvent);
        var time = () => {
            return (new Date().getTime() - this.startDate.getTime()) > this.minimumPressDuration;
        };
        if (this.recognizered || time()) {
            this.handler(GestureRecognizerState.Changed, this.values);
        }
        else if (!this.recognizered && this.validTouch(view) && time()) {
            this.recognizered = true;
            this.handler(GestureRecognizerState.Began, this.values);
        }
        else if (!this.validTouch(view)) {
            this.touchesCancelled(view, touchEvent, undefined);
        }
    }
    touchesEnded(view, touchEvent, args) {
        if (this.canceled) {
            clearTimeout(this.timer);
            this.resetValues();
            return;
        }
        this.updateValues(touchEvent);
        var time = () => {
            return (new Date().getTime() - this.startDate.getTime()) > this.minimumPressDuration;
        };
        if (this.recognizered || (this.validTouch(view) && time())) {
            this.handler(GestureRecognizerState.Ended, this.values);
        }
        clearTimeout(this.timer);
        this.resetValues();
    }
    touchesCancelled(view, touchEvent, args) {
        this.canceled = true;
        this.handler(GestureRecognizerState.Cancelled, this.values);
    }
    resetValues() {
        super.resetValues();
        this.start = undefined;
        this.recognizered = false;
        this.canceled = false;
        this.startDate = new Date();
    }
    validTouch(view) {
        if (this.numberOfTouchesRequired > 1) {
            for (var key in this.values.touches) {
                if (this.values.touches.hasOwnProperty(key)) {
                    var touch = this.values.touches[key];
                    if ((touch.startX + touch.offsetX < view.bounds.width &&
                        touch.startY + touch.offsetY < view.bounds.height &&
                        touch.offsetX > -this.allowableMovement && touch.offsetX < this.allowableMovement &&
                        touch.offsetY > -this.allowableMovement && touch.offsetY < this.allowableMovement) === false) {
                        return false;
                    }
                }
            }
            return true;
        }
        else {
            let fingerID = this.values.touchIDs[this.values.touchIDs.length - 1];
            if (typeof fingerID === "string" || typeof fingerID === "number") {
                let touch = this.values.touches[fingerID];
                if (touch === undefined) {
                    return false;
                }
                return touch.startX + touch.offsetX < view.bounds.width &&
                    touch.startY + touch.offsetY < view.bounds.height &&
                    touch.offsetX > -this.allowableMovement && touch.offsetX < this.allowableMovement &&
                    touch.offsetY > -this.allowableMovement && touch.offsetY < this.allowableMovement;
            }
            else {
                return false;
            }
        }
    }
}
export class PanGestureRecognizer extends GestureRecognizer {
    constructor(handler) {
        super(handler);
        this.responder = RN.PanResponder.create({
            onStartShouldSetPanResponder: (evt, gestureState) => true,
            onStartShouldSetPanResponderCapture: (evt, gestureState) => true,
            onMoveShouldSetPanResponder: (evt, gestureState) => true,
            onMoveShouldSetPanResponderCapture: (evt, gestureState) => true,
            onPanResponderGrant: (event, args) => {
                this.touchesBegan(this.view, event, args);
            },
            onPanResponderMove: (event, args) => {
                this.touchesMoved(this.view, event, args);
            },
            onPanResponderTerminationRequest: (event, gestureState) => true,
            onPanResponderRelease: (event, args) => {
                this.touchesEnded(this.view, event, args);
            },
            onPanResponderTerminate: (event, args) => {
            },
        });
        if (true) {
            this.responder.onShouldBlockNativeResponder = () => true;
        }
    }
    touchesBegan(view, touchEvent, args) {
        this.resetValues();
        this.updateValues(touchEvent);
        this.handler(GestureRecognizerState.Began, this.values);
    }
    touchesMoved(view, touchEvent, args) {
        this.updateValues(touchEvent);
        if (!!args) {
            this.values.touches[touchEvent.nativeEvent.identifier].velocityX = args.vx;
            this.values.touches[touchEvent.nativeEvent.identifier].velocityY = args.vy;
        }
        this.handler(GestureRecognizerState.Changed, this.values);
    }
    touchesEnded(view, touchEvent, args) {
        this.updateValues(touchEvent);
        this.handler(GestureRecognizerState.Ended, this.values);
    }
}
export class SwipeGestureRecognizer extends GestureRecognizer {
    constructor(...args) {
        super(...args);
        this.direction = SwipeGestureRecognizerDirection.Right;
        this.canceled = false;
        this.startTime = 0;
    }
    touchesBegan(view, touchEvent, args) {
        this.resetValues();
        this.updateValues(touchEvent);
        this.handler(GestureRecognizerState.Began, this.values);
    }
    touchesMoved(view, touchEvent, args) {
        if (this.canceled) {
            return;
        }
        this.updateValues(touchEvent);
        let touch = GestureRecognizer.lastTouch(this.values);
        if (touch !== undefined) {
            let shouldCancel = false;
            if (this.direction & SwipeGestureRecognizerDirection.Left || (this.direction & SwipeGestureRecognizerDirection.Right)) {
                if (Math.abs(touch.offsetY) > 128.0) {
                    shouldCancel = true;
                }
                else if ((this.direction & SwipeGestureRecognizerDirection.Left) == 0) {
                    if (touch.offsetX < -4.0) {
                        shouldCancel = true;
                    }
                }
                else if ((this.direction & SwipeGestureRecognizerDirection.Right) == 0) {
                    if (touch.offsetX > 4.0) {
                        shouldCancel = true;
                    }
                }
            }
            if (this.direction & SwipeGestureRecognizerDirection.Up || this.direction & SwipeGestureRecognizerDirection.Down) {
                if (Math.abs(touch.offsetX) > 128.0) {
                    shouldCancel = true;
                }
                else if ((this.direction & SwipeGestureRecognizerDirection.Up) == 0) {
                    if (touch.offsetY > 4.0) {
                        shouldCancel = true;
                    }
                }
                else if ((this.direction & SwipeGestureRecognizerDirection.Down) == 0) {
                    if (touch.offsetY < -4.0) {
                        shouldCancel = true;
                    }
                }
            }
            if (shouldCancel) {
                this.canceled = true;
                this.touchesCancelled(view, touchEvent, args);
            }
        }
    }
    touchesEnded(view, touchEvent, args) {
        if (!this.canceled && (Date.now() - this.startTime) < 250) {
            let touch = GestureRecognizer.lastTouch(this.values);
            if (touch !== undefined) {
                if ((this.direction & SwipeGestureRecognizerDirection.Left || this.direction & SwipeGestureRecognizerDirection.Right)) {
                    if (Math.abs(touch.offsetX) < 44.0) {
                        this.handler(GestureRecognizerState.Failed, this.values);
                    }
                    else {
                        this.handler(GestureRecognizerState.Ended, this.values);
                    }
                }
                if ((this.direction & SwipeGestureRecognizerDirection.Up || this.direction & SwipeGestureRecognizerDirection.Down)) {
                    if (Math.abs(touch.offsetY) < 44.0) {
                        this.handler(GestureRecognizerState.Failed, this.values);
                    }
                    else {
                        this.handler(GestureRecognizerState.Ended, this.values);
                    }
                }
            }
            else {
                this.handler(GestureRecognizerState.Failed, this.values);
            }
        }
        else {
            this.handler(GestureRecognizerState.Failed, this.values);
        }
        this.resetValues();
    }
    touchesCancelled(view, touchEvent, args) {
        this.handler(GestureRecognizerState.Cancelled, this.values);
    }
    resetValues() {
        super.resetValues();
        this.canceled = false;
        this.startTime = Date.now();
    }
}
