/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { ImageRenderingMode, ImageViewContentMode } from "./Enum";
/**
 * Image
 */
export class Image {
    constructor() {
        this.renderingMode = ImageRenderingMode.Original;
    }
    static resource(source) {
        let instance = new Image();
        instance.source = source;
        return instance;
    }
    static remote(uri, sourceSize) {
        let instance = new Image();
        instance.source = { uri: uri };
        instance.sourceSize = sourceSize;
        return instance;
    }
    imageWithRenderingMode(renderingMode) {
        this.renderingMode = renderingMode;
        return this;
    }
}
/**
 * ImageView
 */
export class ImageView extends View {
    imageStyles() {
        let attrs = {};
        Object.assign(attrs, this.props.style);
        if (this.props.image.renderingMode === ImageRenderingMode.Template) {
            attrs.tintColor = this.props.tintColor;
        }
        if (this.props.image.sourceSize !== null && this.props.image.sourceSize !== undefined &&
            this.props.image.sourceSize.width > 0 && this.props.image.sourceSize.height > 0) {
            attrs.width = this.props.image.sourceSize.width;
            attrs.height = this.props.image.sourceSize.height;
        }
        if (this.props.cornerRadius == -1.0) {
            attrs.borderRadius = attrs.height / 2.0;
            attrs.overflow = attrs.borderRadius > 0.0 ? "hidden" : "visible";
        }
        else {
            attrs.borderRadius = this.props.cornerRadius;
            attrs.overflow = attrs.borderRadius > 0.0 ? "hidden" : "visible";
        }
        if (this.props.contentMode == ImageViewContentMode.AspectFill) {
            attrs.overflow = "hidden";
        }
        return attrs;
    }
    resizeMode() {
        switch (this.props.contentMode) {
            case ImageViewContentMode.AspectFill:
                return "cover";
            case ImageViewContentMode.AspectFit:
                return "contain";
            case ImageViewContentMode.ScaleToFill:
                return "stretch";
        }
        return undefined;
    }
    view() {
        if (this.props.image === null) {
            return null;
        }
        return (React.createElement(RN.Image, {source: this.props.image.source, style: this.imageStyles(), resizeMode: this.resizeMode()}, this.props.children));
    }
    render() {
        if (this.props.simpleImage) {
            return this.view();
        }
        else {
            return super.render();
        }
    }
}
ImageView.defaultProps = Object.assign({}, View.defaultProps, {
    contentMode: ImageViewContentMode.AspectFill,
    simpleImage: false,
});
