/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
const buildStyleInterpolator = require('./buildStyleInterpolator');
import { ViewController } from "./ViewController";
import { PixelLine } from "./PixelLine";
import { PresetColor } from "./Types";
import { ScrollView } from "./ScrollView";
import { TabBarController } from "./TabBarController";
/**
 * NavigationBar
 */
export class NavigationBar extends RN.Navigator.NavigationBar {
    scrollsToTopResponder() {
        return {
            onStartShouldSetResponder: () => true,
            onResponderRelease: () => {
                if (this.lastTapDate === undefined || (new Date()).getTime() - this.lastTapDate.getTime() > 300) {
                    this.lastTapDate = new Date();
                }
                else {
                    ScrollView.scrollsToTop(); // Double Tap.
                }
            },
        };
    }
    render() {
        var navBarStyle = {
            height: this.props.navigationStyles.General.TotalNavHeight,
        };
        var navState = this.props.navState;
        var components = navState.routeStack.map((route, index) => NavigationBar.COMPONENT_NAMES.map(componentName => this._getComponent(componentName, route, index)));
        return (React.createElement(RN.View, __assign({key: this._key, style: [
            NavigationBar.styles.navBarContainer,
            navBarStyle,
            { backgroundColor: this.props.barTintColor },
            this.props.style
        ]}, this.scrollsToTopResponder()), components, React.createElement(PixelLine, {color: this.props.shadowColor, style: NavigationBar.styles.aLine})));
    }
}
NavigationBar.defaultProps = {
    tintColor: PresetColor.tintColor,
    barTintColor: "#f9f9f9",
    shadowColor: PresetColor.lineColor,
    titleLabelProps: {},
    backIndicatorImage: null,
};
NavigationBar.COMPONENT_NAMES = ['Title', 'LeftButton', 'RightButton'];
NavigationBar.styles = RN.StyleSheet.create({
    navBarContainer: {
        position: 'absolute',
        top: 0,
        left: 0,
        right: 0,
    },
    aLine: {
        position: 'absolute',
        bottom: 0,
        left: 0,
        right: 0,
        height: 1,
    }
});
/**
 * NavigationController
 */
export class NavigationController extends ViewController {
    constructor(...args) {
        super(...args);
        this.rootViewController = undefined;
        this.currentTitle = "";
        this.currentViewControllers = [];
        this.isActive = true;
        this.isBeingPush = false;
        this.isBeingPop = false;
    }
    viewDidLoad() {
        super.viewDidLoad();
        RN.BackAndroid.addEventListener('hardwareBackPress', () => {
            if (this.isActive && this.navigator.getCurrentRoutes().length > 1) {
                this.popViewController();
                return true;
            }
            return false;
        });
    }
    viewWillAppear() {
        super.viewWillAppear();
        this.currentViewControllers.forEach((vc) => {
            vc.viewWillAppear();
        });
    }
    viewDidAppear() {
        super.viewDidAppear();
        this.currentViewControllers.forEach((vc) => {
            vc.viewDidAppear();
        });
    }
    viewWillDisappear() {
        super.viewWillDisappear();
        this.currentViewControllers.forEach((vc) => {
            vc.viewWillDisappear();
        });
    }
    viewDidDisappear() {
        super.viewDidDisappear();
        this.currentViewControllers.forEach((vc) => {
            vc.viewDidDisappear();
        });
    }
    loadView() {
        let Navigator = RN.Navigator;
        let hasAddListener = false;
        let navigationStyles = NavigationBarStyle();
        let navigationBarProps = this.props.navigationBarProps;
        let marginBottom = typeof this.props.tabBarHeight === "number" ? this.props.tabBarHeight : 0.0;
        return (React.createElement(Navigator, {initialRoute: { component: this.props.rootViewControllerClass, args: this.props.rootViewControllerArgs }, configureScene: (route) => {
            let config = RN.Navigator.SceneConfigs.PushFromRight;
            config.gestures.pop.edgeHitWidth = 88.0;
            config.gestures.jumpForward = null;
            config.animationInterpolators = {
                into: buildStyleInterpolator(NavigationControllerAnimationParams.FromTheRight),
                out: buildStyleInterpolator(NavigationControllerAnimationParams.ToTheLeft),
            };
            return config;
        }, renderScene: (route, navigator) => {
            this.navigator = navigator;
            navigator.navigationController = this;
            if (!hasAddListener) {
                navigator.navigationContext.addListener('willfocus', (event, b) => {
                    if (this.isBeingPush === false && this.isBeingPop === false) {
                        this.isBeingPop = true;
                    }
                    if (this.isBeingPop) {
                        if (this.currentViewControllers.length > 1) {
                            this.currentViewControllers[this.currentViewControllers.length - 1].viewWillDisappear();
                            this.currentViewControllers[this.currentViewControllers.length - 2].viewWillAppear();
                        }
                    }
                    else {
                        this.isBeingPush = true;
                        if (this.currentViewControllers.length > 0) {
                            this.currentViewControllers[this.currentViewControllers.length - 1].viewWillDisappear();
                        }
                    }
                });
                navigator.navigationContext.addListener('didfocus', () => {
                    this.currentTitle = route.component.title;
                    if (this.isBeingPop) {
                        this.isBeingPop = false;
                        this.isBeingPush = false;
                        if (this.currentViewControllers.length > 1) {
                            this.currentViewControllers[this.currentViewControllers.length - 1].viewDidDisappear();
                            this.currentViewControllers[this.currentViewControllers.length - 2].viewDidAppear();
                        }
                        if (this.currentViewControllers.length <= 2) {
                            TabBarController.sharedController && TabBarController.sharedController.setState({
                                hiddingWhilePushing: false
                            });
                        }
                    }
                    else {
                        this.isBeingPush = false;
                        if (this.currentViewControllers.length > 1) {
                            this.currentViewControllers[this.currentViewControllers.length - 1].viewDidAppear();
                            this.currentViewControllers[this.currentViewControllers.length - 2].viewDidDisappear();
                            TabBarController.sharedController && TabBarController.sharedController.setState({
                                hiddingWhilePushing: true
                            });
                        }
                    }
                });
                hasAddListener = true;
            }
            return React.createElement(route.component, __assign({navigationController: this, marginTop: navigationStyles.General.TotalNavHeight, marginBottom: marginBottom}, route.args));
        }, navigationBar: React.createElement(NavigationBar, __assign({navigationStyles: navigationStyles, routeMapper: NavigationController.NavigationBarRouteMapper(this.props)}, navigationBarProps)), translucent: false}));
    }
    showViewController(ViewControllerClass, args) {
        if (this.isBeingPush) {
            setTimeout(() => {
                this.showViewController(ViewControllerClass, args);
            }, 100);
            return;
        }
        this.isBeingPush = true;
        this.navigator.push({
            component: ViewControllerClass,
            args: Object.assign({}, args, { marginBottom: 0.0 }),
        });
    }
    popViewController() {
        this.isBeingPop = true;
        this.navigator.pop();
    }
    popToRootViewController() {
        this.isBeingPop = true;
        this.navigator.popToTop();
    }
}
NavigationController.defaultProps = {
    rootViewControllerClass: ViewController,
    rootViewControllerArgs: {},
    navigationBarProps: {},
};
NavigationController.NavigationBarRouteMapper = (props) => ({
    LeftButton(route, navigator, index, navState) {
        let item = route.component.navigationItem(navigator.navigationController);
        if (index > 0 && item.leftBarButtonItem == null) {
            item.resetBackButtonItem(navigator.navigationController.currentTitle, navigator.navigationController);
            return item.backBarButtonItem;
        }
        else {
            return item.leftBarButtonItem;
        }
    },
    RightButton(route, navigator, index, navState) {
        return route.component.navigationItem(navigator.navigationController).rightBarButtonItem;
    },
    Title(route, navigator, index, navState) {
        return route.component.navigationItem(navigator.navigationController).titleView;
    },
});
class NavigationControllerAnimationParams {
}
NavigationControllerAnimationParams.FromTheRight = {
    opacity: {
        value: 1.0,
        type: 'constant',
    },
    transformTranslate: {
        from: { x: RN.Dimensions.get('window').width, y: 0, z: 0 },
        to: { x: 0, y: 0, z: 0 },
        min: 0,
        max: 1,
        type: 'linear',
        extrapolate: true,
        round: RN.PixelRatio.get(),
    },
    translateX: {
        from: RN.Dimensions.get('window').width,
        to: 0,
        min: 0,
        max: 1,
        type: 'linear',
        extrapolate: true,
        round: RN.PixelRatio.get(),
    },
    scaleX: {
        value: 1,
        type: 'constant',
    },
    scaleY: {
        value: 1,
        type: 'constant',
    },
};
NavigationControllerAnimationParams.ToTheLeft = {
    transformTranslate: {
        from: { x: 0, y: 0, z: 0 },
        to: { x: -RN.Dimensions.get('window').width, y: 0, z: 0 },
        min: 0,
        max: 1,
        type: 'linear',
        extrapolate: true,
        round: RN.PixelRatio.get(),
    },
    opacity: {
        value: 1.0,
        type: 'constant',
    },
    translateX: {
        from: 0,
        to: -RN.Dimensions.get('window').width,
        min: 0,
        max: 1,
        type: 'linear',
        extrapolate: true,
        round: RN.PixelRatio.get(),
    },
};
let NavigationBarStyle = () => {
    let merge = require('merge');
    var SCREEN_WIDTH = RN.Dimensions.get('window').width;
    var NAV_BAR_HEIGHT = RN.Platform.OS === 'android' ? 48 : 44;
    var STATUS_BAR_HEIGHT = RN.Platform.OS === 'android' ? 0 : 20;
    var NAV_HEIGHT = NAV_BAR_HEIGHT + STATUS_BAR_HEIGHT;
    var BASE_STYLES = {
        Title: {
            position: 'absolute',
            top: STATUS_BAR_HEIGHT,
            left: 0,
            right: 0,
            alignItems: 'center',
            height: NAV_BAR_HEIGHT,
            backgroundColor: 'transparent',
        },
        LeftButton: {
            position: 'absolute',
            top: STATUS_BAR_HEIGHT,
            left: 0,
            overflow: 'hidden',
            opacity: 1,
            height: NAV_BAR_HEIGHT,
            backgroundColor: 'transparent',
        },
        RightButton: {
            position: 'absolute',
            top: STATUS_BAR_HEIGHT,
            right: 0,
            overflow: 'hidden',
            opacity: 1,
            alignItems: 'flex-end',
            height: NAV_BAR_HEIGHT,
            backgroundColor: 'transparent',
        },
    };
    // There are 3 stages: left, center, right. All previous navigation
    // items are in the left stage. The current navigation item is in the
    // Another way to think of the stages is in terms of transitions. When
    // we move forward in the navigation stack, we perform a
    // right-to-center transition on the new navigation item and a
    // center-to-left transition on the current navigation item.
    var Stages = {
        Left: {
            Title: merge(BASE_STYLES.Title, { left: -SCREEN_WIDTH / 2, opacity: 0 }),
            LeftButton: merge(BASE_STYLES.LeftButton, { left: -SCREEN_WIDTH / 3, opacity: 0 }),
            RightButton: merge(BASE_STYLES.RightButton, { opacity: 0 }),
        },
        Center: {
            Title: merge(BASE_STYLES.Title, { left: 0, opacity: 1 }),
            LeftButton: merge(BASE_STYLES.LeftButton, { left: 0, opacity: 1 }),
            RightButton: merge(BASE_STYLES.RightButton, { opacity: 1 }),
        },
        Right: {
            Title: merge(BASE_STYLES.Title, { left: SCREEN_WIDTH / 2, opacity: 0 }),
            LeftButton: merge(BASE_STYLES.LeftButton, { left: 0, opacity: 0 }),
            RightButton: merge(BASE_STYLES.RightButton, { opacity: 0 }),
        },
    };
    var opacityRatio = 100;
    function buildSceneInterpolators(startStyles, endStyles) {
        return {
            Title: buildStyleInterpolator({
                opacity: {
                    type: 'linear',
                    from: startStyles.Title.opacity,
                    to: endStyles.Title.opacity,
                    min: 0,
                    max: 1,
                },
                left: {
                    type: 'linear',
                    from: startStyles.Title.left,
                    to: endStyles.Title.left,
                    min: 0,
                    max: 1,
                    extrapolate: true,
                },
            }),
            LeftButton: buildStyleInterpolator({
                opacity: {
                    type: 'linear',
                    from: startStyles.LeftButton.opacity,
                    to: endStyles.LeftButton.opacity,
                    min: 0,
                    max: 1,
                    round: opacityRatio,
                },
                left: {
                    type: 'linear',
                    from: startStyles.LeftButton.left,
                    to: endStyles.LeftButton.left,
                    min: 0,
                    max: 1,
                },
            }),
            RightButton: buildStyleInterpolator({
                opacity: {
                    type: 'linear',
                    from: startStyles.RightButton.opacity,
                    to: endStyles.RightButton.opacity,
                    min: 0,
                    max: 1,
                    round: opacityRatio,
                },
                left: {
                    type: 'linear',
                    from: startStyles.RightButton.left,
                    to: endStyles.RightButton.left,
                    min: 0,
                    max: 1,
                    extrapolate: true,
                },
            }),
        };
    }
    var Interpolators = {
        // Animating *into* the center stage from the right
        RightToCenter: buildSceneInterpolators(Stages.Right, Stages.Center),
        // Animating out of the center stage, to the left
        CenterToLeft: buildSceneInterpolators(Stages.Center, Stages.Left),
        // Both stages (animating *past* the center stage)
        RightToLeft: buildSceneInterpolators(Stages.Right, Stages.Left),
    };
    return {
        General: {
            NavBarHeight: NAV_BAR_HEIGHT,
            StatusBarHeight: STATUS_BAR_HEIGHT,
            TotalNavHeight: NAV_HEIGHT,
        },
        Interpolators: Interpolators,
        Stages: Stages,
    };
};
