/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
/**
 * ProgressView
 */
export class ProgressView extends View {
    constructor(props) {
        super(props);
        this.progress = props.initProgress;
    }
    componentWillReceiveProps(nextProps) {
        super.componentWillReceiveProps(nextProps);
        if (typeof nextProps.currentProgress == "number") {
            setTimeout(() => {
                this.setProgress(nextProps.currentProgress, true);
            });
        }
    }
    setProgress(progress, animated) {
        if (animated) {
            View.setAnimate(() => {
                this.progress = progress;
                this.forceUpdate();
            }, null, 'spring', { friction: 12.0, tension: 40.0, toValue: 0.0 });
        }
        else {
            this.progress = progress;
            this.forceUpdate();
        }
        RN.InteractionManager.runAfterInteractions(() => {
            if (progress === 1.0) {
                this.setProgress(0.0, false);
            }
        });
    }
    layoutSubviews() {
        View.setAnimate(() => {
            this.forceUpdate();
        }, null, 'spring', { friction: 12.0, tension: 40.0, toValue: 0.0 });
    }
    view() {
        return (React.createElement(View, {frame: { x: 0, y: 0, mx: 0, my: 0 }, userInteractionEnabled: false}, React.createElement(View, {frame: { x: 0, y: 0, mx: 0, my: 0 }}), React.createElement(View, {animatable: true, frame: { x: 0, width: this.bounds.width * this.progress, y: 0, my: 0 }, backgroundColor: this.props.tintColor})));
    }
}
ProgressView.defaultProps = Object.assign({}, View.defaultProps, {
    initProgress: 0.5,
    currentProgress: 0.5,
    frame: { height: 2.0 },
    boundsRelative: true,
});
