/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
/**
 * RefreshControl
 */
export class RefreshControl extends View {
    constructor(props) {
        super(props);
        this.state = Object.assign({}, this.state, {
            progress: 0.0,
            refreshing: false,
        });
    }
    customStyle() {
        return {
            position: "absolute",
            left: 0,
            right: 0,
            top: 0,
            height: 200,
        };
    }
    view() {
        return (React.createElement(RN.View, {style: { flex: 1, height: 100.0 }}, React.createElement(ChrysanthemumView, {style: { alignSelf: 'center' }, progress: this.state.progress, animating: this.state.refreshing})));
    }
    update(progress, refreshing) {
        this.setState({
            progress: progress,
            refreshing: refreshing,
        });
    }
}
RefreshControl.triggerHeight = 144.0;
RefreshControl.runningHeight = 54.0;
class ChrysanthemumView extends React.Component {
    constructor(props) {
        super(props);
        this.refreshing = false;
        this.state = {
            currentFrame: 0,
        };
    }
    componentWillReceiveProps(nextProps, nextContext) {
        if (nextProps.animating === true) {
            this.startAnimating();
        }
    }
    startAnimating() {
        if (this.refreshing) {
            return;
        }
        this.update(true);
        this.refreshing = true;
    }
    update(force = false) {
        if (!this.props.animating && !force) {
            this.refreshing = false;
            return;
        }
        let nextFrame = force ? -12 : this.state.currentFrame + 1;
        if (nextFrame > 11) {
            nextFrame = 0;
        }
        this.setState({
            currentFrame: nextFrame
        });
        if (nextFrame < 0) {
            requestAnimationFrame(() => {
                this.update(false);
            });
        }
        else {
            requestAnimationFrame(() => {
                requestAnimationFrame(() => {
                    requestAnimationFrame(() => {
                        this.update(false);
                    });
                });
            });
        }
    }
    render() {
        let opacities = [];
        let rolling = 0;
        if (!this.props.animating) {
            for (var index = 0; index < 12; index++) {
                if (this.props.progress > 0.08 * (index + 1)) {
                    opacities.push(1.0);
                }
                else {
                    opacities.push(0.0);
                }
            }
        }
        else {
            if (this.state.currentFrame < 0) {
                // rolling
                for (var index = 0; index < 12; index++) {
                    opacities.push(1.0);
                }
                rolling = Math.abs(Math.abs(this.state.currentFrame) - 12) / 12 * 60;
            }
            else {
                // animating
                let opacity = 1.0;
                let reveringOpacities = [];
                for (var index = this.state.currentFrame; index < this.state.currentFrame + 12; index++) {
                    if (index >= 12) {
                        reveringOpacities.push(opacity);
                    }
                    else {
                        opacities.push(opacity);
                    }
                    opacity -= 0.07;
                }
                reveringOpacities.reverse();
                reveringOpacities.forEach(element => {
                    opacities.unshift(element);
                });
            }
        }
        return (React.createElement(RN.View, {style: [{ width: 54.0, height: 54.0, transform: [{ scaleX: 0.5 }, { scaleY: 0.5 }, { translateX: 0 }, { translateY: 0 }, { rotate: rolling + 'deg' }] }, this.props.style]}, React.createElement(RN.View, {style: { opacity: opacities[0], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '0deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[1], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '30deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[2], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '60deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[3], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '90deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[4], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '120deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[5], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '150deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[6], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '180deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[7], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '210deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[8], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '240deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[9], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '270deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[10], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '300deg' }, { translateX: 0 }, { translateY: -18 }] }}), React.createElement(RN.View, {style: { opacity: opacities[11], position: 'absolute', top: 42 / 2, left: 50 / 2, width: 8 / 2, height: 26 / 2, borderRadius: 8 / 2, backgroundColor: 'gray', transform: [{ rotate: '330deg' }, { translateX: 0 }, { translateY: -18 }] }})));
    }
}
