/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { ScrollDirection, ScrollViewKeyboardDismissMode } from "./Enum";
let topStacks = [];
/**
 * ScrollView
 */
export class ScrollView extends View {
    constructor(props) {
        super(props);
        this.contentSize = { width: 0, height: 0 };
        this.contentOffset = { x: 0.0, y: 0.0 };
        this.tracking = false;
        this.dragging = false;
        this.decelerating = false;
        this.scrollViewRefreshControlImp = new ScrollViewRefreshControlIMP(this);
        this.scrollViewGestureImp = new ScrollViewGestureIMP(this);
        this.scrollViewLayoutImp = new ScrollViewLayoutImp(this);
    }
    static scrollsToTop() {
        let currentStacks = topStacks;
        let found = false;
        currentStacks.reverse();
        currentStacks.forEach((item) => {
            if (item.props.scrollsToTop && !found) {
                item.setContentOffset({ x: 0, y: 0 }, true);
                found = true;
            }
        });
    }
    setContentOffset(offset, animated) {
        this.scrollViewLayoutImp.setContentOffset(offset, animated);
        this.updateIndicatorView();
        this.props.delegate && this.props.delegate.scrollViewDidScroll && this.props.delegate.scrollViewDidScroll(this);
    }
    setContentInsets(insets, animated) {
        this.scrollViewLayoutImp.setContentInset(insets, animated);
    }
    onTouchStart() { }
    onScroll() {
        this.scrollViewRefreshControlImp.onScroll();
        this.props.delegate && this.props.delegate.scrollViewDidScroll && this.props.delegate.scrollViewDidScroll(this);
    }
    onTouchEnd() {
        this.scrollViewRefreshControlImp.onTouchEnd();
    }
    onScrollToTop() { }
    onRefresh(completionBlock) {
        this.props.onRefresh && this.props.onRefresh(completionBlock);
    }
    componentDidMount() {
        if (this.props.scrollsToTop) {
            topStacks.push(this);
        }
    }
    componentWillUnmount() {
        topStacks = topStacks.filter((val) => {
            return val !== this;
        });
    }
    customStyle() {
        return {
            flex: 1,
        };
    }
    customProps() {
        return this.scrollViewGestureImp.viewProps();
    }
    content() {
        return (React.createElement(RN.View, {style: { flexDirection: this.props.scrollDirection === ScrollDirection.Horizontal ? "row" : "column" }}, this.props.children));
    }
    overlayer() {
        return null;
    }
    view() {
        return (React.createElement(RN.View, {style: { flex: 1 }}, this.scrollViewRefreshControlImp.requestRefreshControl(), React.createElement(RN.Animated.View, {style: {
            marginLeft: this.scrollViewLayoutImp.animatedValues().x,
            marginTop: this.scrollViewLayoutImp.animatedValues().y,
            paddingTop: this.scrollViewLayoutImp.animatedValues().insets.top,
            paddingLeft: this.scrollViewLayoutImp.animatedValues().insets.left,
            paddingBottom: this.scrollViewLayoutImp.animatedValues().insets.bottom,
            paddingRight: this.scrollViewLayoutImp.animatedValues().insets.right,
            backgroundColor: "transparent",
        }, onLayout: (event) => {
            this.contentOffset = { x: -event.nativeEvent.layout.x, y: -event.nativeEvent.layout.y };
            this.contentSize = this.props.contentSize ? this.props.contentSize : { width: event.nativeEvent.layout.width, height: event.nativeEvent.layout.height };
            this.updateIndicatorView();
            this.onScroll();
        }}, React.createElement(RN.Animated.View, {style: {
            position: 'absolute',
            top: this.scrollViewLayoutImp.animatedValues().insets.top,
            left: this.scrollViewLayoutImp.animatedValues().insets.left,
            bottom: this.scrollViewLayoutImp.animatedValues().insets.bottom,
            right: this.scrollViewLayoutImp.animatedValues().insets.right,
            backgroundColor: "white",
        }, pointerEvents: "none"}), this.content()), this.overlayer(), React.createElement(ScrollViewIndicatorView, {ref: "vIndicator", horizontal: false, scrollView: this}), React.createElement(ScrollViewIndicatorView, {ref: "hIndicator", horizontal: true, scrollView: this})));
    }
    showIndicatorView() {
        if (this.props.scrollDirection == ScrollDirection.Vertical || this.props.scrollDirection == ScrollDirection.AllDirection) {
            this.props.showsVerticalScrollIndicator === true && this.refs["vIndicator"] && this.refs["vIndicator"].show();
        }
        if (this.props.scrollDirection == ScrollDirection.Horizontal || this.props.scrollDirection == ScrollDirection.AllDirection) {
            this.props.showsHorizontalScrollIndicator === true && this.refs["hIndicator"] && this.refs["hIndicator"].show();
        }
    }
    updateIndicatorView() {
        this.refs["vIndicator"] && this.refs["vIndicator"].update();
        this.refs["hIndicator"] && this.refs["hIndicator"].update();
    }
    hideIndicatorView(animated = true) {
        this.refs["vIndicator"] && this.refs["vIndicator"].hide(animated);
        this.refs["hIndicator"] && this.refs["hIndicator"].hide(animated);
    }
}
ScrollView.defaultProps = Object.assign({}, View.defaultProps, {
    scrollDirection: ScrollDirection.Vertical,
    scrollEnabled: true,
    bounces: true,
    alwaysBounceVertical: false,
    alwaysBounceHorizontal: false,
    showsHorizontalScrollIndicator: true,
    showsVerticalScrollIndicator: true,
    decelerationRate: 2.0,
    scrollsToTop: false,
    pageEnabled: false,
    clipsToBounds: true,
    contentSize: undefined,
    directionalLockEnabled: false,
    canCancelContentTouches: true,
    keyboardDismissMode: ScrollViewKeyboardDismissMode.OnPress,
});
/**
 * ScrollViewRefreshControlIMP
 */
export class ScrollViewRefreshControlIMP {
    constructor(scrollView) {
        this.scrollView = scrollView;
    }
    get isRefreshing() {
        return this.refreshing;
    }
    get triggerHeight() {
        if (this.scrollView.props.refreshControl !== undefined) {
            return this.scrollView.props.refreshControl.triggerHeight;
        }
        else {
            return 0;
        }
    }
    get runningHeight() {
        if (this.scrollView.props.refreshControl !== undefined) {
            return this.scrollView.props.refreshControl.runningHeight;
        }
        else {
            return 0;
        }
    }
    requestRefreshControl() {
        if (this.scrollView.props.refreshControl !== undefined) {
            let c = { type: this.scrollView.props.refreshControl };
            return React.createElement(c.type, {ref: "RefreshControl", scrollView: this.scrollView});
        }
        else {
            return null;
        }
    }
    onScroll() {
        if (!this.scrollView.dragging) {
            return;
        }
        if (this.scrollView.contentOffset.y > 0.0) {
            return;
        }
        let currentValue = this.scrollView.contentOffset.y;
        let progress = 0.0;
        if (currentValue <= 0.0) {
            progress = Math.max(-this.triggerHeight, currentValue) / -this.triggerHeight;
        }
        let refreshControl = this.scrollView.refs["RefreshControl"];
        if (refreshControl !== undefined) {
            if (progress == 1.0 && !this.refreshing) {
                this.refreshing = true;
            }
            refreshControl.update(progress, this.refreshing);
        }
    }
    onTouchEnd() {
        if (this.scrollView.contentOffset.y > 44.0) {
            return;
        }
        let refreshControl = this.scrollView.refs["RefreshControl"];
        if (this.refreshing && refreshControl !== undefined) {
            this.scrollView.setContentInsets({ top: this.runningHeight }, true);
            this.scrollView.onRefresh(() => {
                this.refreshing = false;
                this.scrollView.setContentInsets({ top: 0.0 }, this.scrollView.contentOffset.y <= this.runningHeight);
                refreshControl.update(0.0, this.refreshing);
            });
        }
    }
}
/**
 * ScrollViewGesture
 */
export class ScrollViewGestureIMP {
    constructor(scrollView) {
        this.historyValues = [];
        this.currentDirection = undefined;
        this.lastVeloctiy = {};
        this.scrollView = scrollView;
        this.scrollViewPanResponder = RN.PanResponder.create(this.panHandlers());
    }
    viewProps() {
        return this.scrollViewPanResponder.panHandlers;
    }
    panHandlers() {
        return {
            onStartShouldSetPanResponder: (event, state) => {
                if (event.nativeEvent.pageX < this.scrollView.props.backTouchingArea) {
                    return false;
                }
                return this.scrollView.props.scrollEnabled;
            },
            onStartShouldSetPanResponderCapture: (event, state) => {
                if (this.scrollView.decelerating) {
                    this.scrollView.scrollViewLayoutImp.stop();
                    this.scrollView.decelerating = false;
                    if (event.nativeEvent.pageX > this.scrollView.props.backTouchingArea) {
                        return true;
                    }
                }
                View.currentTouchResponder = this.scrollView;
                return false;
            },
            onMoveShouldSetPanResponder: (event, state) => {
                if (event.nativeEvent.pageX < this.scrollView.props.backTouchingArea) {
                    return false;
                }
                if (this.scrollView.props.scrollEnabled && this.scrollView.props.canCancelContentTouches) {
                    View.nextTouchResponder = this.scrollView;
                    return true;
                }
                else {
                    return false;
                }
            },
            onPanResponderGrant: (event, state) => {
                this.touchesStart();
            },
            onPanResponderMove: (event, state) => {
                this.touchesMoved(event, state);
            },
            onPanResponderTerminationRequest: (event, state) => {
                return false;
            },
            onPanResponderRelease: (event, state) => {
                View.nextTouchResponder = undefined;
                this.touchesEnded(event, state);
            },
            onPanResponderTerminate: (event, state) => {
                this.touchesEnded(event, state);
            },
        };
    }
    touchesStart() {
        this.scrollView.onTouchStart();
        this.scrollView.tracking = true;
        if (this.scrollView.props.keyboardDismissMode === ScrollViewKeyboardDismissMode.OnPress) {
            View.endEditing();
        }
        this.historyValues = [];
        this.scrollView.scrollViewLayoutImp.start();
    }
    touchesMoved(touchEvent, state) {
        if (!this.scrollView.dragging) {
            this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewWillBeginDragging && this.scrollView.props.delegate.scrollViewWillBeginDragging(this.scrollView);
        }
        this.scrollView.dragging = true;
        let values = this.requestValues(state);
        this.scrollView.scrollViewLayoutImp.move(values.distance, values.velocity);
        this.scrollView.showIndicatorView();
    }
    touchesEnded(touchEvent, state) {
        this.scrollView.onTouchEnd();
        this.scrollView.tracking = false;
        this.scrollView.dragging = false;
        let values = this.requestValues(state);
        this.historyValues.push(values);
        this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewWillEndDragging && this.scrollView.props.delegate.scrollViewWillEndDragging(this.scrollView, values.velocity);
        this.scrollView.scrollViewLayoutImp.end(values.distance, values.velocity);
        this.currentDirection = undefined;
    }
    requestValues(state) {
        let distance = {
            top: state.dy > 0.0 ? Math.abs(state.dy) : undefined,
            left: state.dx > 0.0 ? Math.abs(state.dx) : undefined,
            bottom: state.dy < 0.0 ? Math.abs(state.dy) : undefined,
            right: state.dx < 0.0 ? Math.abs(state.dx) : undefined,
        };
        let velocity = {
            top: state.vy > 0.0 ? Math.abs(state.vy) : undefined,
            left: state.vx > 0.0 ? Math.abs(state.vx) : undefined,
            bottom: state.vy < 0.0 ? Math.abs(state.vy) : undefined,
            right: state.vx < 0.0 ? Math.abs(state.vx) : undefined,
        };
        if (this.scrollView.props.directionalLockEnabled === true && this.scrollView.props.scrollDirection == ScrollDirection.AllDirection) {
            if (this.currentDirection === undefined) {
                // Not determine
                if (distance.top > 10.0 || distance.bottom > 10.0) {
                    this.currentDirection = ScrollDirection.Vertical;
                }
                else if (distance.left > 10.0 || distance.right > 10.0) {
                    this.currentDirection = ScrollDirection.Horizontal;
                }
                else {
                    return { distance: {}, velocity: {} };
                }
            }
        }
        if (RN.Platform.OS == "android") {
            // android 滑动改进
            velocity.top *= 2;
            velocity.bottom *= 2;
            velocity.left *= 2;
            velocity.right *= 2;
        }
        if (this.scrollView.props.scrollDirection == ScrollDirection.Horizontal || this.currentDirection == ScrollDirection.Horizontal) {
            delete distance.top, delete distance.bottom, delete velocity.top, delete velocity.bottom;
        }
        if (this.scrollView.props.scrollDirection == ScrollDirection.Vertical || this.currentDirection == ScrollDirection.Vertical) {
            delete distance.left, delete distance.right, delete velocity.left, delete velocity.right;
        }
        if (this.scrollView.props.pageEnabled === true) {
            if (distance.top >= this.scrollView.bounds.height) {
                distance.top = this.scrollView.bounds.height;
                velocity.top = 0.0;
            }
            if (distance.bottom >= this.scrollView.bounds.height) {
                distance.bottom = this.scrollView.bounds.height;
                velocity.bottom = 0.0;
            }
            if (distance.left >= this.scrollView.bounds.width) {
                distance.left = this.scrollView.bounds.width;
                velocity.left = 0.0;
            }
            if (distance.right >= this.scrollView.bounds.width) {
                distance.right = this.scrollView.bounds.width;
                velocity.right = 0.0;
            }
        }
        return {
            distance: distance,
            velocity: velocity,
        };
    }
}
/**
 * ScrollViewLayoutImp
 */
export class ScrollViewLayoutImp {
    constructor(scrollView) {
        this.scrollViewOffsetX = new RN.Animated.Value(0.0);
        this.scrollViewOffsetY = new RN.Animated.Value(0.0);
        this.scrollViewInsets = {
            top: new RN.Animated.Value(0.0),
            left: new RN.Animated.Value(0.0),
            bottom: new RN.Animated.Value(0.0),
            right: new RN.Animated.Value(0.0),
        };
        this.hideIndicatorViewTimer = undefined;
        this.scrollView = scrollView;
    }
    animatedValues() {
        return {
            x: this.scrollViewOffsetX,
            y: this.scrollViewOffsetY,
            insets: this.scrollViewInsets,
        };
    }
    contentOffset(callback) {
        callback(this.scrollView.contentOffset);
    }
    setContentOffset(offset, animated) {
        if (animated) {
            if (typeof offset.x === "number") {
                RN.Animated.spring(this.scrollViewOffsetX, {
                    toValue: -offset.x,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (typeof offset.y === "number") {
                RN.Animated.spring(this.scrollViewOffsetY, {
                    toValue: -offset.y,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (typeof offset.x === "number" && typeof offset.y === "number" && offset.x === 0 && offset.y === 0) {
                setTimeout(() => {
                    this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewDidScrollToTop && this.scrollView.props.delegate.scrollViewDidScrollToTop(this.scrollView);
                    this.scrollView.onScrollToTop();
                }, 250);
            }
        }
        else {
            if (typeof offset.x === "number") {
                this.scrollViewOffsetX.setValue(-offset.x);
            }
            if (typeof offset.y === "number") {
                this.scrollViewOffsetY.setValue(-offset.y);
            }
            if (typeof offset.x === "number" && typeof offset.y === "number" && offset.x === 0 && offset.y === 0) {
                this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewDidScrollToTop && this.scrollView.props.delegate.scrollViewDidScrollToTop(this.scrollView);
                this.scrollView.onScrollToTop();
            }
        }
    }
    setContentInset(insets, animated) {
        if (animated) {
            if (typeof insets.top === "number") {
                RN.Animated.spring(this.scrollViewInsets.top, {
                    toValue: insets.top,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (typeof insets.left === "number") {
                RN.Animated.spring(this.scrollViewInsets.left, {
                    toValue: insets.left,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (typeof insets.bottom === "number") {
                RN.Animated.spring(this.scrollViewInsets.bottom, {
                    toValue: insets.bottom,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (typeof insets.right === "number") {
                RN.Animated.spring(this.scrollViewInsets.right, {
                    toValue: insets.right,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
        }
        else {
            if (typeof insets.top === "number") {
                this.scrollViewInsets.top.setValue(insets.top);
            }
            if (typeof insets.left === "number") {
                this.scrollViewInsets.left.setValue(insets.left);
            }
            if (typeof insets.bottom === "number") {
                this.scrollViewInsets.bottom.setValue(insets.bottom);
            }
            if (typeof insets.right === "number") {
                this.scrollViewInsets.right.setValue(insets.right);
            }
        }
    }
    start() {
        this.stop();
        this.startOffset = { x: this.scrollView.contentOffset.x, y: this.scrollView.contentOffset.y };
    }
    move(distance, velocity) {
        let moveTo = ScrollViewAlgorithm.moveTo(this.startOffset, distance);
        if (ScrollViewAlgorithm.outOfBounds(this.scrollView.bounds, this.scrollView.contentSize, moveTo)) {
            if (!this.scrollView.props.bounces) {
                // 禁止边界回弹
                let backPoint = ScrollViewAlgorithm.pointNearByBounds(this.scrollView.bounds, this.scrollView.contentSize, this.scrollView.contentOffset);
                let xAllow = this.scrollView.props.scrollDirection == ScrollDirection.AllDirection || this.scrollView.props.scrollDirection == ScrollDirection.Horizontal || this.scrollView.scrollViewGestureImp.currentDirection == ScrollDirection.Horizontal;
                let yAllow = this.scrollView.props.scrollDirection == ScrollDirection.AllDirection || this.scrollView.props.scrollDirection == ScrollDirection.Vertical || this.scrollView.scrollViewGestureImp.currentDirection == ScrollDirection.Vertical;
                if (typeof backPoint.x === "number" && xAllow) {
                    this.scrollViewOffsetX.setValue(backPoint.x);
                }
                if (typeof backPoint.y === "number" && yAllow) {
                    this.scrollViewOffsetY.setValue(backPoint.y);
                }
            }
            else {
                // 边界阻力加成
                let outLength = ScrollViewAlgorithm.outLengthOfBounds(this.scrollView.bounds, this.scrollView.contentSize, moveTo);
                let margin = ScrollViewAlgorithm.margin(ScrollViewAlgorithm.moveResistance(outLength, this.startOffset, distance));
                this.scrollViewOffsetX.setValue(margin.x);
                this.scrollViewOffsetY.setValue(margin.y);
            }
        }
        else {
            this.scrollViewOffsetX.setValue(ScrollViewAlgorithm.margin(moveTo).x);
            this.scrollViewOffsetY.setValue(ScrollViewAlgorithm.margin(moveTo).y);
        }
        this.hideIndicatorViewTimer && clearTimeout(this.hideIndicatorViewTimer);
    }
    end(distance, velocity) {
        this.stop();
        if (this.scrollView.bounds.height <= 0.0) {
            return;
        }
        if (ScrollViewAlgorithm.outOfBounds(this.scrollView.bounds, this.scrollView.contentSize, this.scrollView.contentOffset)) {
            this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewWillBeginDecelerating && this.scrollView.props.delegate.scrollViewWillBeginDecelerating(this.scrollView);
            this.scrollView.decelerating = true;
            // back bounces (边界回弹).
            // Warning: Apple Inc.'s Patent, UXKit use for study.
            let backPoint = ScrollViewAlgorithm.pointNearByBounds(this.scrollView.bounds, this.scrollView.contentSize, this.scrollView.contentOffset);
            let xAllow = this.scrollView.props.scrollDirection == ScrollDirection.AllDirection || this.scrollView.props.scrollDirection == ScrollDirection.Horizontal || this.scrollView.scrollViewGestureImp.currentDirection == ScrollDirection.Horizontal;
            let yAllow = this.scrollView.props.scrollDirection == ScrollDirection.AllDirection || this.scrollView.props.scrollDirection == ScrollDirection.Vertical || this.scrollView.scrollViewGestureImp.currentDirection == ScrollDirection.Vertical;
            if (typeof backPoint.x === "number" && xAllow) {
                RN.Animated.spring(this.scrollViewOffsetX, {
                    toValue: ScrollViewAlgorithm.margin(backPoint).x,
                    friction: 24.0,
                    tension: 200.0,
                }).start();
            }
            if (typeof backPoint.y === "number" && yAllow) {
                RN.Animated.spring(this.scrollViewOffsetY, {
                    toValue: ScrollViewAlgorithm.margin(backPoint).y,
                    friction: 24.0,
                    tension: 200.0,
                }).start();
            }
        }
        else if (this.scrollView.props.pageEnabled === true) {
            // page bounce (分页推动)
            if (this.scrollView.props.scrollDirection === ScrollDirection.Horizontal || this.scrollView.props.scrollDirection === ScrollDirection.AllDirection) {
                RN.Animated.spring(this.scrollViewOffsetX, {
                    toValue: ScrollViewAlgorithm.margin(ScrollViewAlgorithm.pointOfPage(distance, velocity, this.scrollView.bounds, this.scrollView.contentSize, this.scrollView.contentOffset)).x,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
            if (this.scrollView.props.scrollDirection === ScrollDirection.Vertical || this.scrollView.props.scrollDirection === ScrollDirection.AllDirection) {
                RN.Animated.spring(this.scrollViewOffsetY, {
                    toValue: ScrollViewAlgorithm.margin(ScrollViewAlgorithm.pointOfPage(distance, velocity, this.scrollView.bounds, this.scrollView.contentSize, this.scrollView.contentOffset)).y,
                    bounciness: 1.0,
                    speed: 100.0,
                }).start();
            }
        }
        else {
            // decay bounce 衰减回弹 
            // Warning: Apple Inc.'s Patent, UXKit use for study.
            let ready = this.scrollView.contentSize.height > 1 && this.scrollView.bounds.height > 1;
            if ((velocity.top > 0 || velocity.bottom > 0) && ready) {
                this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewWillBeginDecelerating && this.scrollView.props.delegate.scrollViewWillBeginDecelerating(this.scrollView);
                this.scrollView.decelerating = true;
                this.scrollViewOffsetY.animate(new DecayBounceAnimation({
                    velocity: velocity.top > 0 ? velocity.top : -velocity.bottom,
                    deceleration: 0.997,
                    minBounce: -(this.scrollView.contentSize.height - this.scrollView.bounds.height),
                    maxBounce: 0,
                }, this.scrollView, this.scrollViewOffsetY));
            }
            if ((velocity.left > 0 || velocity.right > 0) && ready) {
                this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewWillBeginDecelerating && this.scrollView.props.delegate.scrollViewWillBeginDecelerating(this.scrollView);
                this.scrollView.decelerating = true;
                this.scrollViewOffsetX.animate(new DecayBounceAnimation({
                    velocity: velocity.left > 0 ? velocity.left : -velocity.right,
                    deceleration: 0.997,
                    minBounce: -(this.scrollView.contentSize.width - this.scrollView.bounds.width),
                    maxBounce: 0,
                }, this.scrollView, this.scrollViewOffsetX));
            }
        }
        RN.InteractionManager.runAfterInteractions(() => {
            this.scrollView.hideIndicatorView();
            this.scrollView.decelerating = false;
            this.scrollView.props.delegate && this.scrollView.props.delegate.scrollViewDidEndDecelerating && this.scrollView.props.delegate.scrollViewDidEndDecelerating(this.scrollView);
        });
    }
    stop() {
        this.scrollViewOffsetX.stopAnimation();
        this.scrollViewOffsetY.stopAnimation();
    }
}
/**
 * ScrollViewIndicatorView
 */
class ScrollViewIndicatorView extends View {
    constructor(...args) {
        super(...args);
        this.iOffset = new RN.Animated.Value(0.0);
        this.iHeight = new RN.Animated.Value(0.0);
        this.iAlpha = new RN.Animated.Value(0.0);
    }
    customStyle() {
        if (!this.props.horizontal) {
            return {
                right: 0,
                top: 0,
                bottom: 0,
                width: 3,
            };
        }
        else {
            return {
                left: 0,
                right: 0,
                bottom: 0,
                height: 3,
            };
        }
    }
    show() {
        RN.Animated.timing(this.iAlpha, {
            toValue: 1.0,
            duration: 300.0,
        }).start();
    }
    hide(animated = true) {
        if (animated) {
            RN.Animated.timing(this.iAlpha, {
                toValue: 0.0,
                duration: 300.0,
            }).start();
        }
        else {
            this.iAlpha.setValue(0.0);
        }
    }
    update() {
        if (this.props.scrollView.bounds.height <= 0.0 || this.props.scrollView.bounds.width <= 0.0 || this.props.scrollView.contentSize.height <= 0.0 || this.props.scrollView.contentSize.width <= 0.0) {
            return;
        }
        let page = 0;
        let iHeight = 0;
        let iOffset = 0;
        let minHeight = 0;
        if (!this.props.horizontal) {
            if (this.props.scrollView.contentSize.height <= this.props.scrollView.bounds.height) {
                this.hide();
                return;
            }
            page = this.props.scrollView.contentSize.height / this.props.scrollView.bounds.height;
            iHeight = page > 0 ? this.props.scrollView.bounds.height / page : 0.0;
            iHeight = Math.max(11.0, iHeight);
            iOffset = this.props.scrollView.contentOffset.y / this.props.scrollView.contentSize.height * this.props.scrollView.bounds.height;
            minHeight = iHeight * 0.2;
            if (iOffset < 0.0) {
                iOffset = (Math.abs(Math.pow(iOffset, 2)) < (iHeight - minHeight)) ? -(Math.abs(Math.pow(iOffset, 2))) : -(iHeight - minHeight);
            }
            else if (iOffset > this.props.scrollView.bounds.height - iHeight) {
                let dVal = this.props.scrollView.bounds.height - iHeight - iOffset;
                let fVal = (Math.abs(Math.pow(dVal, 2)) < (iHeight - minHeight)) ? (Math.abs(Math.pow(dVal, 2))) : (iHeight - minHeight);
                iOffset = this.props.scrollView.bounds.height - iHeight + fVal;
            }
        }
        else {
            if (this.props.scrollView.contentSize.width <= this.props.scrollView.bounds.width) {
                this.hide();
                return;
            }
            page = this.props.scrollView.contentSize.width / this.props.scrollView.bounds.width;
            iHeight = page > 0 ? this.props.scrollView.bounds.width / page : 0.0;
            iHeight = Math.max(11.0, iHeight);
            iOffset = this.props.scrollView.contentOffset.x / this.props.scrollView.contentSize.width * this.props.scrollView.bounds.width;
            minHeight = iHeight * 0.2;
            if (iOffset < 0.0) {
                iOffset = (Math.abs(Math.pow(iOffset, 2)) < (iHeight - minHeight)) ? -(Math.abs(Math.pow(iOffset, 2))) : -(iHeight - minHeight);
            }
            else if (iOffset > this.props.scrollView.bounds.width - iHeight) {
                let dVal = this.props.scrollView.bounds.width - iHeight - iOffset;
                let fVal = (Math.abs(Math.pow(dVal, 2)) < (iHeight - minHeight)) ? (Math.abs(Math.pow(dVal, 2))) : (iHeight - minHeight);
                iOffset = this.props.scrollView.bounds.width - iHeight + fVal;
            }
        }
        this.iHeight.setValue(iHeight);
        this.iOffset.setValue(iOffset);
    }
    view() {
        if (!this.props.horizontal) {
            return (React.createElement(RN.Animated.View, {style: { backgroundColor: "#a2a2a2e0", marginTop: this.iOffset, height: this.iHeight, width: 2, opacity: this.iAlpha, borderRadius: 2.0 }}));
        }
        else {
            return (React.createElement(RN.Animated.View, {style: { backgroundColor: "#a2a2a2e0", marginLeft: this.iOffset, width: this.iHeight, height: 2, opacity: this.iAlpha }}));
        }
    }
    shouldComponentUpdate() {
        return true;
    }
}
ScrollViewIndicatorView.defaultProps = Object.assign({}, View.defaultProps, {
    style: { position: "absolute" }
});
/**
 * ScrollViewAlgorithm
 */
export class ScrollViewAlgorithm {
    // 边界检测
    static outOfBounds(scrollViewBounds, contentSize, contentOffset) {
        return contentOffset.x < 0 || contentOffset.y < 0 || contentOffset.x + scrollViewBounds.width > contentSize.width || contentOffset.y + scrollViewBounds.height > contentSize.height;
    }
    static decayValue(fromValue, velocity, deceleration) {
        return fromValue + (velocity / (1 - deceleration)) * (1 - Math.exp(-(1 - deceleration) * 16000));
    }
    // 边界超出计算
    static outLengthOfBounds(scrollViewBounds, contentSize, contentOffset) {
        let result = {};
        if (contentOffset.x < 0) {
            result.left = -contentOffset.x;
        }
        if (contentOffset.y < 0) {
            result.top = -contentOffset.y;
        }
        if (contentOffset.x + scrollViewBounds.width > contentSize.width) {
            result.right = contentOffset.x + scrollViewBounds.width - contentSize.width;
        }
        if (contentOffset.y + scrollViewBounds.height > contentSize.height) {
            result.bottom = contentOffset.y + scrollViewBounds.height - contentSize.height;
        }
        return result;
    }
    // 最近边界
    static pointNearByBounds(scrollViewBounds, contentSize, contentOffset) {
        let aPoint = {};
        if (contentOffset.x < 0) {
            aPoint.x = 0;
        }
        if (contentOffset.y < 0) {
            aPoint.y = 0;
        }
        if (contentOffset.x + scrollViewBounds.width > contentSize.width) {
            aPoint.x = contentSize.width - scrollViewBounds.width;
        }
        if (contentOffset.y + scrollViewBounds.height > contentSize.height) {
            aPoint.y = contentSize.height - scrollViewBounds.height;
        }
        return aPoint;
    }
    // 分页推进坐标点计算
    static pointOfPage(distance, velocity, scrollViewBounds, contentSize, contentOffset) {
        let point = { x: 0, y: 0 };
        if (distance.left > 0) {
            if (velocity.left > 0) {
                point.x = this.pointOfCurrentPage(scrollViewBounds, contentOffset).x;
            }
            else {
                point.x = contentOffset.x % scrollViewBounds.width < scrollViewBounds.width * 4 / 5 ? this.pointOfCurrentPage(scrollViewBounds, contentOffset).x : this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).x;
            }
        }
        else if (distance.right > 0) {
            if (velocity.right > 0) {
                point.x = this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).x;
            }
            else {
                point.x = contentOffset.x % scrollViewBounds.width > scrollViewBounds.width * 1 / 5 ? this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).x : this.pointOfCurrentPage(scrollViewBounds, contentOffset).x;
            }
        }
        if (distance.top > 0) {
            if (velocity.top > 0) {
                point.y = this.pointOfCurrentPage(scrollViewBounds, contentOffset).y;
            }
            else {
                point.y = contentOffset.y % scrollViewBounds.height < scrollViewBounds.height * 4 / 5 ? this.pointOfCurrentPage(scrollViewBounds, contentOffset).y : this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).y;
            }
        }
        else if (distance.bottom > 0) {
            if (velocity.bottom > 0) {
                point.y = this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).y;
            }
            else {
                point.y = contentOffset.y % scrollViewBounds.height > scrollViewBounds.height * 1 / 5 ? this.pointOfNextPage(scrollViewBounds, contentSize, contentOffset).y : this.pointOfCurrentPage(scrollViewBounds, contentOffset).y;
            }
        }
        return point;
    }
    // 上一页坐标点
    static pointOfPrevPage(scrollViewBounds, contentOffset) {
        let x = scrollViewBounds.width * (Math.floor(contentOffset.x / scrollViewBounds.width) - 1);
        let y = scrollViewBounds.height * (Math.floor(contentOffset.y / scrollViewBounds.height) - 1);
        if (x <= 0.0) {
            x = 0.0;
        }
        if (y <= 0.0) {
            y = 0.0;
        }
        return { x: x, y: y };
    }
    // 下一页坐标点
    static pointOfCurrentPage(scrollViewBounds, contentOffset) {
        return {
            x: scrollViewBounds.width * (Math.floor(contentOffset.x / scrollViewBounds.width) + 0),
            y: scrollViewBounds.height * (Math.floor(contentOffset.y / scrollViewBounds.height) + 0),
        };
    }
    // 下一页坐标点
    static pointOfNextPage(scrollViewBounds, contentSize, contentOffset) {
        let x = scrollViewBounds.width * (Math.floor(contentOffset.x / scrollViewBounds.width) + 1);
        let y = scrollViewBounds.height * (Math.floor(contentOffset.y / scrollViewBounds.height) + 1);
        if (x >= contentSize.width) {
            x = scrollViewBounds.width * Math.floor(contentOffset.x / scrollViewBounds.width);
        }
        if (y >= contentSize.height) {
            y = scrollViewBounds.height * Math.floor(contentOffset.y / scrollViewBounds.height);
        }
        return { x: x, y: y };
    }
    // 将 contentOffset 转换为 margin
    static margin(aPoint) {
        let margin = {};
        if (typeof aPoint.x === "number") {
            margin.x = -aPoint.x;
        }
        if (typeof aPoint.y === "number") {
            margin.y = -aPoint.y;
        }
        return margin;
    }
    // 线性计算
    static moveTo(startPoint, distance) {
        let toOffset = { x: startPoint.x, y: startPoint.y };
        if (distance.top > 0) {
            toOffset.y -= distance.top;
        }
        else if (distance.bottom > 0) {
            toOffset.y += distance.bottom;
        }
        if (distance.left > 0) {
            toOffset.x -= distance.left;
        }
        else if (distance.right > 0) {
            toOffset.x += distance.right;
        }
        return toOffset;
    }
    // 阻力计算
    static moveResistance(outLength, startPoint, distance) {
        let toOffset = { x: startPoint.x, y: startPoint.y };
        if (distance.top > 0 && outLength.top > 0) {
            toOffset.y -= distance.top;
            toOffset.y += outLength.top;
            toOffset.y -= this.resistance(outLength.top);
        }
        else if (distance.bottom > 0 && outLength.bottom > 0) {
            toOffset.y += distance.bottom;
            toOffset.y -= outLength.bottom;
            toOffset.y += this.resistance(outLength.bottom);
        }
        if (distance.left > 0 && outLength.left > 0) {
            toOffset.x -= distance.left;
            toOffset.x += outLength.left;
            toOffset.x -= this.resistance(outLength.left);
        }
        else if (distance.right > 0 && outLength.right > 0) {
            toOffset.x += distance.right;
            toOffset.x -= outLength.right;
            toOffset.x += this.resistance(outLength.right);
        }
        return toOffset;
    }
    // 阻力值 (Log函数)
    static resistance(value) {
        let originValue = value;
        let resistanceValue = Math.log(value * value * value * value * value) / Math.log(1.20) - 1;
        return resistanceValue > originValue ? originValue : resistanceValue;
    }
    // 增强阻力值
    static strongResistance(value) {
        let originValue = value;
        let resistanceValue = Math.log(value * value * value * value) / Math.log(1.40) - 1;
        return resistanceValue > originValue ? originValue : resistanceValue;
    }
}
class DecayBounceAnimation {
    constructor(config, scrollView, animatedValue) {
        this.scrollView = scrollView;
        this.animatedValue = animatedValue;
        this.deceleration = config.deceleration !== undefined ? config.deceleration : 0.997;
        this.velocity = config.velocity;
        this.minBounce = config.minBounce;
        this.maxBounce = config.maxBounce;
    }
    start(fromValue, updateHandler, endHandler) {
        this.active = true;
        this.lastValue = fromValue;
        this.fromValue = fromValue;
        this.updateHandler = updateHandler;
        this.endHandler = endHandler;
        this.startTime = Date.now();
        this.animationFrame = requestAnimationFrame(this.onUpdate.bind(this));
    }
    onUpdate() {
        if (!this.active) {
            return;
        }
        let now = Date.now();
        let value = this.fromValue + (this.velocity / (1 - this.deceleration)) * (1 - Math.exp(-(1 - this.deceleration) * (now - this.startTime)));
        let minLimitValue = (this.fromValue + (this.velocity / (1 - this.deceleration)) * (1 - Math.exp(-(1 - this.deceleration) * 50000)));
        let minMiddleValue = (this.minBounce - minLimitValue) / 2.0;
        let maxLimitValue = (this.fromValue + (this.velocity / (1 - this.deceleration)) * (1 - Math.exp(-(1 - this.deceleration) * 50000)));
        let maxMiddleValue = (maxLimitValue - this.maxBounce) / 2.0;
        if (value > this.maxBounce) {
            if (Math.abs(value - maxLimitValue) < maxMiddleValue) {
                RN.Animated.spring(this.animatedValue, {
                    toValue: this.maxBounce,
                    bounciness: 1.0,
                    velocity: Math.abs(this.velocity) * 2
                }).start();
                return;
            }
            let dValue = value - this.maxBounce;
            let rValue = ScrollViewAlgorithm.strongResistance(dValue);
            let fValue = this.maxBounce + rValue;
            this.updateHandler(fValue);
            if (this.active) {
                this.animationFrame = requestAnimationFrame(this.onUpdate.bind(this));
            }
        }
        else if (value < this.minBounce) {
            if (Math.abs(value - minLimitValue) < minMiddleValue) {
                RN.Animated.spring(this.animatedValue, {
                    toValue: this.minBounce,
                    bounciness: 1.0,
                    velocity: Math.abs(this.velocity) * 2
                }).start();
                return;
            }
            let dValue = this.minBounce - value;
            let rValue = ScrollViewAlgorithm.strongResistance(dValue);
            let fValue = this.minBounce - rValue;
            this.updateHandler(fValue);
            if (this.active) {
                this.animationFrame = requestAnimationFrame(this.onUpdate.bind(this));
            }
        }
        else {
            this.updateHandler(value);
            if (Math.abs(this.lastValue - value) < 1.0) {
                this.scrollView.decelerating = false;
            }
            if (Math.abs(this.lastValue - value) > 0.1 && this.active) {
                this.animationFrame = requestAnimationFrame(this.onUpdate.bind(this));
            }
            else {
                this.endHandler && this.endHandler();
            }
        }
        this.lastValue = value;
    }
    stop() {
        this.active = false;
        window.cancelAnimationFrame(this.animationFrame);
        this.endHandler && this.endHandler();
    }
}
