/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { RectangleButton, TextButton } from "./Button";
import { ControlEvents } from "./Enum";
import { PresetColor } from "./Types";
/**
 * Switch
 */
export class SegmentedControl extends View {
    constructor(props) {
        super(props);
        this.selectedIndex = props.initSelectedIndex;
    }
    view() {
        return (React.createElement(RN.View, {style: { flex: 1, flexDirection: 'row' }}, this.buttons()));
    }
    customStyle() {
        return {
            flex: 0,
            borderRadius: 6.0,
            borderWidth: 1.0,
            borderColor: this.props.tintColor,
        };
    }
    buttons() {
        let buttons = [];
        for (var i = 0; i < this.props.buttonTitles.length; i++) {
            var buttonTitle = this.props.buttonTitles[i];
            var selected = this.selectedIndex == i ? true : false;
            buttons.push(React.createElement(RectangleButton, {tintColor: this.props.tintColor, style: function () {
                if (i == 0) {
                    return {
                        flex: 1,
                        borderTopLeftRadius: 5.0,
                        borderBottomLeftRadius: 5.0,
                    };
                }
                else if (i == this.props.buttonTitles.length - 1) {
                    return {
                        flex: 1,
                        borderTopRightRadius: 5.0,
                        borderBottomRightRadius: 5.0,
                    };
                }
                else {
                    return {
                        flex: 1,
                    };
                }
            }.bind(this)(), highlightedColor: this.props.tintColorDisabled, borderWidth: 0, cornerRadius: 0, tag: i, key: "b_" + i, title: buttonTitle, selected: selected, handler: (event, sender) => {
                if (event == ControlEvents.TouchUpInside) {
                    this.selectedIndex = sender.props.tag;
                    this.props.handler(sender.props.tag, this);
                    this.forceUpdate();
                }
            }}));
            if (i + 1 < this.props.buttonTitles.length) {
                buttons.push(React.createElement(View, {key: "s_" + i, style: { width: 1, backgroundColor: this.props.tintColor }}));
            }
        }
        return buttons;
    }
}
SegmentedControl.defaultProps = Object.assign({}, View.defaultProps, {
    initSelectedIndex: 0,
    buttonTitles: [],
    handler: (selectedIndex, sender) => { },
    frame: { height: 32.0 }
});
/**
 * SegmentedControl_Style2
 */
export class SegmentedControl_Style2 extends SegmentedControl {
    constructor(...args) {
        super(...args);
        this.updating = false;
    }
    updateWithAnimation() {
        this.updating = true;
        View.setAnimate(() => {
            this.setState({});
        }, null, 'spring', { friction: 20.0, tension: 120.0, toValue: 0.0 }, () => { this.updating = false; });
    }
    layoutSubviews() {
        this.updating || this.forceUpdate();
    }
    view() {
        return (React.createElement(RN.View, {style: { flex: 1, flexDirection: 'row' }}, this.buttons(), React.createElement(View, {animatable: true, frame: {
            x: this.bounds.width / this.props.buttonTitles.length * this.selectedIndex,
            y: this.bounds.height - 1.0,
            width: this.bounds.width / this.props.buttonTitles.length,
            height: 1
        }, backgroundColor: this.props.tintColor})));
    }
    customStyle() {
        return {
            flex: 0,
        };
    }
    buttons() {
        var buttons = [];
        for (var i = 0; i < this.props.buttonTitles.length; i++) {
            var buttonTitle = this.props.buttonTitles[i];
            var selected = this.selectedIndex == i ? true : false;
            buttons.push(React.createElement(TextButton, {key: i, style: { flex: 1 }, tag: i, color: selected ? this.props.tintColor : PresetColor.secondTextColor, title: buttonTitle, selected: selected, handler: (event, sender) => {
                if (event == ControlEvents.TouchUpInside) {
                    this.selectedIndex = sender.props.tag;
                    this.props.handler(sender.props.tag, this);
                    this.updateWithAnimation();
                }
            }}));
        }
        return buttons;
    }
}
SegmentedControl_Style2.defaultProps = Object.assign({}, SegmentedControl.defaultProps, {
    frame: { height: 38.0 },
    boundsRelative: true,
});
