/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { GestureRecognizerState } from "./Enum";
/**
 * Slider
 */
export class Slider extends View {
    constructor(props) {
        super(props);
        this.numberValue = 0.0;
        this.backTintWidthValue = new RN.Animated.Value(0.0);
        this.buttonLeftValue = new RN.Animated.Value(0.0);
        this.trackValue = 0.0;
        this.numberValue = props.initValue;
        this.trackValue = this.numberValue;
        this.panGesture = RN.PanResponder.create({
            onStartShouldSetPanResponder: (evt, gestureState) => true,
            onStartShouldSetPanResponderCapture: (evt, gestureState) => true,
            onMoveShouldSetPanResponder: (evt, gestureState) => true,
            onMoveShouldSetPanResponderCapture: (evt, gestureState) => true,
            onPanResponderGrant: (evt, gestureState) => {
                this.handlePan(GestureRecognizerState.Began, null);
            },
            onPanResponderMove: (evt, gestureState) => {
                this.handlePan(GestureRecognizerState.Changed, gestureState.dx);
            },
            onPanResponderTerminationRequest: (evt, gestureState) => true,
            onPanResponderRelease: (evt, gestureState) => {
                this.handlePan(GestureRecognizerState.Ended, gestureState.dx);
            },
            onPanResponderTerminate: (evt, gestureState) => {
                this.handlePan(GestureRecognizerState.Cancelled, gestureState.dx);
            }
        });
        if (true) {
            (this.panGesture).onShouldBlockNativeResponder = () => true;
        }
    }
    value() {
        return this.numberValue;
    }
    setValue(value, animated) {
        this.numberValue = value;
        this.resetViews(animated);
    }
    componentDidMount() {
        this.numberValue = this.props.initValue;
        this.resetViews(false);
    }
    resetViews(animated) {
        if (animated) {
            RN.Animated.spring(this.backTintWidthValue, {
                toValue: this.numberValue * (this.bounds.width - 30.0),
                friction: 20.0,
                tension: 80.0,
            }).start();
            RN.Animated.spring(this.buttonLeftValue, {
                toValue: this.bounds.width < 1.0 ? -1000 : this.numberValue * (this.bounds.width - 44.0),
                friction: 20.0,
                tension: 80.0,
            }).start();
        }
        else {
            this.backTintWidthValue.setValue(this.numberValue * (this.bounds.width - 30.0));
            this.buttonLeftValue.setValue(this.bounds.width < 1.0 ? -1000 : this.numberValue * (this.bounds.width - 44.0));
        }
    }
    componentWillReceiveProps(nextProps) {
        super.componentWillReceiveProps(nextProps);
        if (typeof nextProps.currentValue == "number") {
            this.numberValue = nextProps.currentValue;
            this.resetViews(false);
        }
    }
    layoutSubviews() {
        super.layoutSubviews();
        console.log(this.bounds);
        this.resetViews(false);
    }
    view() {
        let handlers = this.panGesture.panHandlers;
        return (React.createElement(View, {style: { flex: 1 }}, React.createElement(View, {style: { position: "absolute", left: 15, right: 15, top: 17.5, height: 9.0, backgroundColor: "#e5e5e5", borderRadius: 6.0 }}), React.createElement(RN.Animated.View, {style: {
            position: "absolute", left: 15, width: this.backTintWidthValue,
            top: 17.5, height: 9.0, backgroundColor: this.props.tintColor, borderRadius: 6.0
        }}), React.createElement(RN.Animated.View, __assign({style: {
            position: "absolute",
            left: this.buttonLeftValue,
            top: 0.0,
            height: 44.0,
            width: 44.0,
        }}, handlers), React.createElement(View, {style: {
            marginLeft: 7.0,
            marginTop: 7.0,
            width: 30.0,
            height: 30.0,
            backgroundColor: "white",
            borderRadius: 15.0,
            borderWidth: 0.75,
            borderColor: "#e5e5e5",
        }}))));
    }
    handlePan(state, x) {
        if (state == GestureRecognizerState.Began) {
            this.trackValue = this.numberValue;
        }
        else if (state == GestureRecognizerState.Changed) {
            let step = x / (this.bounds.width - 30.0);
            this.numberValue = Math.max(Math.min(this.trackValue + step, 1.0), 0.0);
            this.resetViews(false);
            this.props.handler(this.numberValue);
        }
        else if (state == GestureRecognizerState.Cancelled || state == GestureRecognizerState.Ended || state == GestureRecognizerState.Failed) {
            let step = x / (this.bounds.width - 30.0);
            this.numberValue = Math.max(Math.min(this.trackValue + step, 1.0), 0.0);
            this.resetViews(false);
            this.props.handler(this.numberValue);
        }
    }
}
Slider.defaultProps = Object.assign({}, View.defaultProps, {
    enabled: true,
    initValue: 0.5,
    currentValue: 0.5,
    handler: (newValue) => { },
    frame: { height: 44.0 },
    boundsRelative: true,
});
