/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import { Control } from "./Control";
import { View } from "./View";
import { ControlEvents } from "./Enum";
/**
 * Switch
 */
export class Switch extends Control {
    constructor(props) {
        super(props);
        this.switchCanceled = false;
        this.state = Object.assign({}, this.state, {
            on: false,
            holding: false,
        });
    }
    isOn() {
        return this.state.on;
    }
    setOn(on, animated) {
        if (animated) {
            View.setAnimate(() => {
                this.setState({
                    on: on,
                });
            }, null, 'spring', { friction: 20.0, tension: 120.0, toValue: 0.0 });
        }
        else {
            this.setState({
                on: on,
            });
        }
    }
    componentWillReceiveProps(newProps) {
        super.componentWillReceiveProps(newProps);
        if (typeof newProps.currentOn === "boolean") {
            setTimeout(() => {
                this.setOn(newProps.currentOn, true);
            });
        }
    }
    content() {
        return (React.createElement(View, {frame: { x: 2.0, y: 6.0, mx: 2.0, height: 32.0 }, backgroundColor: "white", cornerRadius: 18.0, borderWidth: 1.5, borderColor: "#e5e5e5", alpha: this.props.enabled ? 1.0 : 0.6}, React.createElement(View, {frame: { x: -1.5, y: -1.5, width: 54, height: 32 }, backgroundColor: this.props.tintColor, cornerRadius: 18.0, alpha: this.state.on ? 1.0 : 0.0, animatable: true}), React.createElement(View, {frame: { x: -1.5, y: -1.5, width: 54, height: 32 }, backgroundColor: "#e5e5e5", cornerRadius: 18.0, alpha: this.state.holding && !this.state.on ? 1.0 : 0.0, animatable: true}), React.createElement(View, {frame: { height: 10.0, width: 10.0, x: 34, y: 10 }, backgroundColor: "white", cornerRadius: 5.0, borderWidth: 0.75, borderColor: "#b2b2b2", hidden: this.state.on || this.state.holding}), React.createElement(View, {frame: { height: 10.0, width: 1.0, x: 12, y: 10 }, backgroundColor: "white", hidden: !this.state.on || this.state.holding}), React.createElement(View, {frame: {
            x: (this.state.on ? (this.state.holding ? 13.0 : 21.0) : -0.5),
            y: -0.5,
            height: 30.0,
            width: (this.state.holding ? 38.0 : 30.0)
        }, backgroundColor: "white", cornerRadius: 15.0, borderWidth: 0.75, borderColor: "#e5e5e5", animatable: true})));
    }
    eventHandler(event) {
        if (event == ControlEvents.TouchDown) {
            this.switchCanceled = false;
            setTimeout(() => {
                if (!this.switchCanceled) {
                    View.setAnimate(() => {
                        this.setState({
                            holding: true
                        });
                    }, null, 'spring', { friction: 20.0, tension: 200.0, toValue: 0.0 });
                }
            }, 100);
        }
        else if (event == ControlEvents.TouchDragEnter) {
            View.setAnimate(() => {
                this.setState({
                    holding: true
                });
            }, null, 'spring', { friction: 20.0, tension: 200.0, toValue: 0.0 });
        }
        else if (event == ControlEvents.TouchDragExit) {
            View.setAnimate(() => {
                this.setState({
                    holding: false
                });
            }, null, 'spring', { friction: 20.0, tension: 200.0, toValue: 0.0 });
        }
        else if (event == ControlEvents.TouchUpInside) {
            this.switchCanceled = true;
            View.setAnimate(() => {
                this.setState({
                    on: !this.state.on,
                    holding: false,
                });
            }, null, 'spring', { friction: 20.0, tension: 120.0, toValue: 0.0 });
            this.props.handler(ControlEvents.ValueChanged, this);
        }
        else if (event == ControlEvents.TouchCancel || event == ControlEvents.TouchUpOutside) {
            this.switchCanceled = true;
            View.setAnimate(() => {
                this.setState({
                    holding: false
                });
            }, null, 'spring', { friction: 20.0, tension: 200.0, toValue: 0.0 });
        }
    }
}
Switch.defaultProps = Object.assign({}, Control.defaultProps, {
    initOn: false,
    currentOn: false,
    moveAllow: 44.0,
    frame: { width: 58.0, height: 44.0 }
});
