/// <reference path="../../typings/tsd.d.ts"/>
import * as RN from "react-native";
let _ = require("lodash");
import { Font } from "./Font";
import { TextAlignment } from "./Enum";
export class TextParagraphStyle {
    textAlign() {
        switch (this.alignment) {
            case TextAlignment.Left:
                return "left";
            case TextAlignment.Center:
                return "center";
            case TextAlignment.Right:
                return "right";
        }
        return "left";
    }
    styles() {
        return {
            textAlign: this.textAlign(),
            lineHeight: this.lineHeight,
        };
    }
}
/**
 * Text
 */
export class Text extends RN.Text {
    static measure(items, width, completionBlock) {
        const measureParams = items.map((item) => {
            return this.requestMeasureParams(item);
        });
        const manager = require('react-native').NativeModules.TextMeasureManager;
        if (undefined != manager && null != manager) {
            manager.measure(measureParams, width, completionBlock);
        }
    }
    static requestMeasureParams(text) {
        return {
            text: text.props.children,
            fontSize: text.props.font.pointSize,
            fontFamily: text.props.font.familyName,
            fontWeight: text.props.font.fontWeight,
            fontStyle: text.props.font.fontStyle,
            letterSpacing: text.props.kern,
            lineHeight: text.props.paragraph != undefined ? text.props.paragraph.lineHeight : undefined,
        };
    }
    render() {
        let props = _.clone(this.props);
        var propStyle = props.style ? props.style : {};
        if (!!this.props.font) {
            Object.assign(propStyle, this.props.font.styles());
        }
        if (!!this.props.paragraph) {
            Object.assign(propStyle, this.props.paragraph.styles());
        }
        if (!!this.props.foregroundColor) {
            Object.assign(propStyle, { color: this.props.foregroundColor });
        }
        if (!!this.props.backgroundColor) {
            Object.assign(propStyle, { backgroundColor: this.props.backgroundColor });
        }
        if (typeof this.props.kern === "number") {
            Object.assign(propStyle, { letterSpacing: this.props.kern });
        }
        if (typeof this.props.underline === "boolean") {
            Object.assign(propStyle, { textDecorationLine: this.props.underline ? "underline" : "none" });
            if (this.props.foregroundColor !== null) {
                Object.assign(propStyle, { textDecorationColor: this.props.foregroundColor });
            }
        }
        if (typeof this.props.link === "string" && typeof this.props.linkHandler === "function") {
            props.onPress = () => {
                let link = this.props.link;
                if (typeof link === "string") {
                    this.props.linkHandler(link);
                }
            };
        }
        props.style = propStyle;
        this.props = props;
        return super.render();
    }
}
Text.defaultProps = {
    font: Font.fontWithSize(15.0),
    paragraph: undefined,
    foregroundColor: "black",
    backgroundColor: "#ffffff00",
    kern: undefined,
    underline: false,
    link: undefined,
    linkHandler: (link) => { },
};
