/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { TextButton } from "./Button";
import { Image } from "./ImageView";
import { Font } from "./Font";
import { TextFieldBorderStyle, TextFieldViewMode, TextAlignment, ControlEvents, ImageRenderingMode } from "./Enum";
import { PresetColor } from "./Types";
import { TextInputTraitsProps } from "./TextInputTraits";
/**
 * TextFieldView
 */
export class TextFieldView extends View {
}
TextFieldView.defaultProps = Object.assign({}, View.defaultProps, {
    mode: TextFieldViewMode.WhileEditing,
    width: 22.0,
    handler: (event) => { },
});
class UXTextFieldClearView extends TextFieldView {
    view() {
        return (React.createElement(TextButton, {image: Image.resource(require("./Assets/ux_clear.png")).imageWithRenderingMode(ImageRenderingMode.Template), tintColor: PresetColor.thirdTextColor, frame: { width: 44 }, contentHorizontalAlignment: "right", contentVerticalAlignment: "center", style: { flex: 1, paddingRight: 8 }, handler: this.props.handler}));
    }
}
/**
 * TextField
 */
export class TextField extends View {
    constructor(props) {
        super(props);
        this.text = "";
        this.editing = false;
        this.text = props.initText;
    }
    becomeFirstResponder() {
        this.refs["TextInput"].focus();
    }
    resignFirstResponder() {
        this.refs["TextInput"].blur();
    }
    clear() {
        this.refs["TextInput"].clear();
    }
    backgroundStyle() {
        return {
            cornerRadius: this.props.borderStyle == 3 ? 4.0 : 0.0,
            borderColor: PresetColor.lineColor,
            borderWidth: this.props.borderStyle == 1 || this.props.borderStyle == 3 ? 1.0 : 0.0,
        };
    }
    textAlign() {
        switch (this.props.textAlignment) {
            case TextAlignment.Left:
                return "left";
            case TextAlignment.Center:
                return "center";
            case TextAlignment.Right:
                return "right";
        }
        return "left";
    }
    textFieldStyle() {
        return {
            alignItems: "center",
            justifyContent: "center",
            color: this.props.textColor,
            textAlign: this.textAlign(),
            paddingLeft: 8.0,
            paddingRight: 8.0,
            marginTop: (RN.Platform.OS == "android" && RN.Platform.Version < 21) ? -4.0 : 0.0,
            height: RN.Platform.OS == "android" ? 38.0 : 30.0,
        };
    }
    textFieldProps() {
        return Object.assign({
            placeholderTextColor: this.props.placeholderColor,
            underlineColorAndroid: "transparent",
            multiline: false,
            onChangeText: this.onTextChange.bind(this),
            onFocus: this.onBeginEditing.bind(this),
            onBlur: this.onEndEditing.bind(this),
            onSubmitEditing: this.onReturn.bind(this),
            enablesReturnKeyAutomatically: true,
        }, TextInputTraitsProps(this.props));
    }
    view() {
        let style = this.textFieldStyle();
        let leftView = this.leftView();
        style.marginLeft = leftView !== null ? leftView.props.width : 0.0;
        let rightView = this.rightView();
        style.marginRight = rightView !== null ? rightView.props.width : 0.0;
        return (React.createElement(View, {frame: { x: 0, y: 0, mx: 0, my: 0 }}, React.createElement(View, __assign({frame: { x: 0, y: 0, mx: 0, height: 30 }}, this.backgroundStyle()), React.createElement(View, {frame: { x: 0, y: 0, my: 0 }, style: { flexDirection: "row" }}, leftView), React.createElement(View, {frame: { mx: 0, y: 0, my: 0 }, style: { flexDirection: "row" }}, rightView)), React.createElement(RN.TextInput, __assign({style: [style, this.props.font.styles()], editable: this.props.enabled, defaultValue: this.props.initText, placeholder: this.props.placeholderText, maxLength: this.props.maxLength, ref: "TextInput"}, this.textFieldProps()))));
    }
    // Views
    leftView() {
        if (this.props.leftView !== null) {
            let leftView = this.props.leftView;
            let show = (leftView.props.mode === TextFieldViewMode.Always) ||
                (leftView.props.mode === TextFieldViewMode.WhileEditing && this.isFirstResponder && this.text.length > 0) ||
                (leftView.props.mode === TextFieldViewMode.UnlessEditing && !this.isFirstResponder && this.text.length > 0);
            return show ? leftView : null;
        }
        else {
            return null;
        }
    }
    rightView() {
        if (this.props.rightView === null) {
            let clearView = React.createElement(UXTextFieldClearView, {handler: (event) => {
                event == ControlEvents.TouchUpInside && this.onClear();
            }});
            let show = (this.props.clearMode === TextFieldViewMode.Always) ||
                (this.props.clearMode === TextFieldViewMode.WhileEditing && this.isFirstResponder && this.text.length > 0) ||
                (this.props.clearMode === TextFieldViewMode.UnlessEditing && !this.isFirstResponder && this.text.length > 0);
            return show ? clearView : null;
        }
        else {
            let rightView = this.props.rightView;
            let show = (rightView.props.mode === TextFieldViewMode.Always) ||
                (rightView.props.mode === TextFieldViewMode.WhileEditing && this.isFirstResponder && this.text.length > 0) ||
                (rightView.props.mode === TextFieldViewMode.UnlessEditing && !this.isFirstResponder && this.text.length > 0);
            return show ? rightView : null;
        }
    }
    // events
    onTextChange(text) {
        this.text = text;
        this.props.handler.didChange && this.props.handler.didChange(text);
        this.setState({});
    }
    onBeginEditing() {
        View.firstResponder = this;
        if (this.props.clearsOnBeginEditing) {
            this.text = "";
            this.clear();
        }
        this.editing = true;
        this.props.handler.didBeginEditing && this.props.handler.didBeginEditing();
        this.isFirstResponder = true;
        this.setState({});
    }
    onEndEditing() {
        this.editing = false;
        this.props.handler.didEndEditing && this.props.handler.didEndEditing();
        this.isFirstResponder = false;
        this.setState({});
    }
    onClear() {
        if (this.props.handler.shouldClear && this.props.handler.shouldClear()) {
            this.text = "";
            this.clear();
        }
        else {
            this.text = "";
            this.clear();
        }
        this.setState({});
    }
    onReturn() {
        if (this.props.handler.shouldReturn && this.props.handler.shouldReturn()) {
            this.resignFirstResponder();
        }
        else {
            this.resignFirstResponder();
        }
    }
}
TextField.defaultProps = Object.assign({}, View.defaultProps, {
    // Additional Views
    leftView: null,
    rightView: null,
    clearMode: TextFieldViewMode.WhileEditing,
    // Styles
    font: Font.fontWithSize(15.0),
    textColor: PresetColor.mainTextColor,
    placeholderColor: PresetColor.thirdTextColor,
    textAlignment: TextAlignment.Left,
    borderStyle: TextFieldBorderStyle.RoundedRect,
    // Props
    enabled: true,
    initText: "",
    placeholderText: "",
    clearsOnBeginEditing: false,
    maxLength: null,
    handler: {},
    frame: { height: RN.Platform.OS == "android" ? 38.0 : 30.0 },
});
