/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
import { PresetColor } from "./Types";
import { PanGestureRecognizer } from "./GestureRecognizer";
let _animatingEnabled = false;
let _animationType = "";
let _animationConfig = {};
export class ViewAnimateImp {
    constructor(obj) {
        this.animationValues = {};
        this.obj = obj;
    }
    static setAnimate(stateBlock, duration = 250, aniType = "timing", animationConfig, completionBlock) {
        _animationType = aniType;
        if (aniType === "timing" && !!!animationConfig) {
            let config = {
                toValue: 0.0,
                duration: duration,
            };
            _animationConfig = config;
        }
        else {
            _animationConfig = animationConfig;
        }
        _animatingEnabled = true;
        stateBlock();
        if (typeof completionBlock === "function") {
            RN.InteractionManager.runAfterInteractions(completionBlock);
        }
        setTimeout(() => {
            _animatingEnabled = false;
        });
    }
    static setLayoutAnimate(stateBlock, duration = 250, aniType = "timing", animationConfig, completionBlock) {
        RN.UIManager.setLayoutAnimationEnabledExperimental && RN.UIManager.setLayoutAnimationEnabledExperimental(true);
        if (aniType === "spring") {
            let ani = RN.LayoutAnimation.Presets.spring;
            ani.duration = duration;
            if (!!animationConfig) {
                ani.update = animationConfig;
            }
            RN.LayoutAnimation.configureNext(ani);
        }
        else {
            let ani = RN.LayoutAnimation.Presets.linear;
            ani.duration = duration;
            if (!!animationConfig) {
                ani.update = animationConfig;
            }
            RN.LayoutAnimation.configureNext(ani);
        }
        stateBlock();
        if (typeof completionBlock === "function") {
            RN.InteractionManager.runAfterInteractions(completionBlock);
        }
    }
    resetAnimationValues(newProps) {
        if (typeof newProps === "undefined") {
            newProps = this.obj.props;
        }
        this.obj.layoutImp.resetAnimationValues(newProps);
        this.obj.drawImp.resetAnimationValues(newProps);
    }
    animateAnimationValues(newProps) {
        if (!_animatingEnabled) {
            return this.resetAnimationValues(newProps);
        }
        var func = _animationType == "spring" ? RN.Animated.spring : RN.Animated.timing;
        this.obj.layoutImp.animateAnimationValues(newProps, func);
        this.obj.drawImp.animateAnimationValues(newProps, func);
    }
}
export class ViewLayoutImp {
    constructor(obj) {
        this.obj = obj;
        this.createAnimationValues();
    }
    createAnimationValues() {
        for (let key in this.obj.props.frame) {
            let value = typeof this.obj.props.frame[key] === "number" ? this.obj.props.frame[key] : 0.0;
            this.obj.animateImp.animationValues["frame." + key] = new RN.Animated.Value(value);
        }
        for (let key in this.obj.props.margin) {
            let value = typeof this.obj.props.margin[key] === "number" ? this.obj.props.margin[key] : 0.0;
            this.obj.animateImp.animationValues["margin." + key] = new RN.Animated.Value(value);
        }
        for (let key in this.obj.props.padding) {
            let value = typeof this.obj.props.padding[key] === "number" ? this.obj.props.padding[key] : 0.0;
            this.obj.animateImp.animationValues["padding." + key] = new RN.Animated.Value(value);
        }
        for (let key in this.obj.props.transform) {
            let value = typeof this.obj.props.transform[key] === "number" ? this.obj.props.transform[key] : 0.0;
            this.obj.animateImp.animationValues["transform." + key] = new RN.Animated.Value(value);
        }
    }
    resetAnimationValues(newProps) {
        if (!!!newProps) {
            newProps = this.obj.props;
        }
        if (!!newProps.frame) {
            for (let key in newProps.frame) {
                let value = newProps.frame[key];
                if (typeof value === "number") {
                    if (key == "cx" || key == "cy") {
                        value = value * 2;
                    }
                    this.obj.animateImp.animationValues["frame." + key] && this.obj.animateImp.animationValues["frame." + key].setValue(value);
                }
            }
        }
        if (!!newProps.margin) {
            for (let key in newProps.margin) {
                if (typeof newProps.margin[key] === "number") {
                    this.obj.animateImp.animationValues["margin." + key] && this.obj.animateImp.animationValues["margin." + key].setValue(newProps.margin[key]);
                }
            }
        }
        if (!!newProps.padding) {
            for (let key in newProps.padding) {
                if (typeof newProps.padding[key] === "number") {
                    this.obj.animateImp.animationValues["padding." + key] && this.obj.animateImp.animationValues["padding." + key].setValue(newProps.padding[key]);
                }
            }
        }
        if (!!newProps.transform) {
            for (let key in newProps.transform) {
                if (typeof newProps.transform[key] === "number") {
                    this.obj.animateImp.animationValues["transform." + key] && this.obj.animateImp.animationValues["transform." + key].setValue(newProps.transform[key]);
                }
            }
        }
    }
    animateAnimationValues(newProps, func) {
        if (!!newProps.frame) {
            for (let key in newProps.frame) {
                if (typeof newProps.frame[key] !== "number") {
                    continue;
                }
                let value = newProps.frame[key];
                if (key == "cx" || key == "cy") {
                    value = value * 2;
                }
                func(this.obj.animateImp.animationValues["frame." + key], Object.assign(_animationConfig, {
                    toValue: value,
                })).start();
            }
        }
        if (!!newProps.margin) {
            for (let key in newProps.margin) {
                if (typeof newProps.margin[key] !== "number") {
                    continue;
                }
                func(this.obj.animateImp.animationValues["margin." + key], Object.assign(_animationConfig, {
                    toValue: newProps.margin[key],
                })).start();
            }
        }
        if (!!newProps.padding) {
            for (let key in newProps.padding) {
                if (typeof newProps.padding[key] !== "number") {
                    continue;
                }
                func(this.obj.animateImp.animationValues["padding." + key], Object.assign(_animationConfig, {
                    toValue: newProps.padding[key],
                })).start();
            }
        }
        if (!!newProps.transform) {
            for (let key in newProps.transform) {
                if (typeof newProps.transform[key] !== "number") {
                    continue;
                }
                func(this.obj.animateImp.animationValues["transform." + key], Object.assign(_animationConfig, {
                    toValue: newProps.transform[key],
                })).start();
            }
        }
    }
    wrapStyles() {
        let styles = {
            flexDirection: 'row',
            justifyContent: typeof this.obj.props.frame.cx === "number" ? 'center' : undefined,
            alignItems: typeof this.obj.props.frame.cy === "number" ? 'center' : undefined,
            position: 'absolute',
            left: 0,
            right: 0,
            top: 0,
            bottom: 0,
        };
        if (typeof this.obj.props.frame.x === "number") {
            styles.justifyContent = "flex-start";
        }
        else if (typeof this.obj.props.frame.mx === "number") {
            styles.justifyContent = "flex-end";
        }
        if (typeof this.obj.props.frame.y === "number") {
            styles.alignItems = "flex-start";
        }
        else if (typeof this.obj.props.frame.my === "number") {
            styles.alignItems = "flex-end";
        }
        return styles;
    }
    viewStyles() {
        let styles = {};
        if (!!this.obj.props.frame) {
            styles.position = (typeof this.obj.props.frame.x === "number" ||
                typeof this.obj.props.frame.y === "number" ||
                typeof this.obj.props.frame.mx === "number" ||
                typeof this.obj.props.frame.my === "number") ? "absolute" : "relative";
            if (typeof this.obj.props.frame.cx === "number" || typeof this.obj.props.frame.cy === "number") {
                styles.position = "relative";
            }
        }
        let frameMap = {
            x: "left",
            y: "top",
            mx: "right",
            my: "bottom",
            width: "width",
            height: "height",
            cx: "marginLeft",
            cy: "marginTop",
        };
        for (let key in this.obj.props.frame) {
            if (typeof this.obj.props.frame[key] === "number") {
                let value = this.obj.props.frame[key];
                if (key == "cx" || key == "cy") {
                    value = value * 2;
                }
                styles[frameMap[key]] = this.obj.props.animatable ? this.obj.animateImp.animationValues["frame." + key] : value;
            }
        }
        if (!!this.obj.props.margin) {
            if (!!this.obj.props.frame && (typeof this.obj.props.frame.cx === "number" || typeof this.obj.props.frame.cy === "number")) { }
            else {
                let marginMap = {
                    top: "marginTop",
                    left: "marginLeft",
                    bottom: "marginBottom",
                    right: "marginRight",
                };
                for (let key in this.obj.props.margin) {
                    if (typeof this.obj.props.margin[key] === "number") {
                        let value = this.obj.props.margin[key];
                        styles[marginMap[key]] = this.obj.props.animatable ? this.obj.animateImp.animationValues["margin." + key] : value;
                    }
                }
            }
        }
        let paddingMap = {
            top: "paddingTop",
            left: "paddingLeft",
            bottom: "paddingBottom",
            right: "paddingRight",
        };
        for (let key in this.obj.props.padding) {
            if (typeof this.obj.props.padding[key] === "number") {
                let value = this.obj.props.padding[key];
                styles[key] = this.obj.props.animatable ? this.obj.animateImp.animationValues["padding." + key] : value;
            }
        }
        let transformMap = {
            scaleX: "scaleX",
            scaleY: "scaleY",
            rotate: "rotate",
        };
        let transform = [];
        for (let key in this.obj.props.transform) {
            if (typeof this.obj.props.transform[key] === "number") {
                let value = this.obj.props.transform[key];
                let item = {};
                if (key == "rotate") {
                    let stringValue = value.toString() + "deg";
                    item[transformMap[key]] = this.obj.props.animatable ? this.obj.animateImp.animationValues["transform." + key].interpolate({
                        inputRange: [0, 360],
                        outputRange: ['0deg', '360deg'],
                    }) : value;
                }
                else {
                    item[transformMap[key]] = this.obj.props.animatable ? this.obj.animateImp.animationValues["transform." + key] : value;
                }
                transform.push(item);
            }
        }
        styles.transform = transform;
        return styles;
    }
    viewProps() {
        return {
            onLayout: this.onLayout.bind(this),
        };
    }
    onLayout(event) {
        if (this.obj.origin.x === event.nativeEvent.layout.x && this.obj.origin.y === event.nativeEvent.layout.y && this.obj.bounds.width === event.nativeEvent.layout.width && this.obj.bounds.height === event.nativeEvent.layout.height) {
            return;
        }
        this.obj.origin = { x: event.nativeEvent.layout.x, y: event.nativeEvent.layout.y };
        this.obj.bounds = { width: event.nativeEvent.layout.width, height: event.nativeEvent.layout.height };
        if (this.obj.props.boundsRelative) {
            this.obj.layoutSubviews();
        }
    }
}
export class ViewDrawImp {
    constructor(obj) {
        this.obj = obj;
        this.createAnimationValues();
    }
    createAnimationValues() {
        if (typeof this.obj.props.alpha === "number") {
            this.obj.animateImp.animationValues["alpha"] = new RN.Animated.Value(this.obj.props.alpha);
        }
        else {
            this.obj.animateImp.animationValues["alpha"] = new RN.Animated.Value(1.0);
        }
    }
    resetAnimationValues(newProps) {
        if (!!newProps && typeof newProps.alpha === "number") {
            this.obj.animateImp.animationValues["alpha"] && this.obj.animateImp.animationValues["alpha"].setValue(newProps.alpha);
        }
    }
    animateAnimationValues(newProps, func) {
        if (!!newProps && typeof newProps.alpha === "number") {
            func(this.obj.animateImp.animationValues["alpha"], Object.assign(_animationConfig, { toValue: newProps.alpha })).start();
        }
    }
    viewStyles() {
        return {
            opacity: this.obj.props.hidden ? 0.0 : (this.obj.props.animatable ? this.obj.animateImp.animationValues["alpha"] : this.obj.props.alpha),
            overflow: this.obj.props.clipsToBounds ? "hidden" : "visible",
            borderRadius: this.obj.props.cornerRadius,
            borderColor: this.obj.props.borderColor,
            borderWidth: this.obj.props.borderWidth,
            backgroundColor: this.obj.props.backgroundColor,
        };
    }
    viewProps() {
        return {
            pointerEvents: (this.obj.props.hidden || this.obj.props.alpha <= 0.001 || !this.obj.props.userInteractionEnabled) ? "none" : "auto",
        };
    }
}
export class ViewTouchImp {
    constructor(obj) {
        this.obj = obj;
    }
    viewProps() {
        if (!!this.obj.props.gestureRecognizers && this.obj.props.gestureRecognizers.length > 0) {
            let firstObject = this.obj.props.gestureRecognizers[0];
            if (firstObject instanceof PanGestureRecognizer) {
                firstObject.view = this.obj;
                return firstObject.responder.panHandlers;
            }
            else {
                return {
                    onStartShouldSetResponder: (event) => {
                        if (View.currentTouchResponder instanceof View) {
                            if (typeof this.obj.props.backTouchingArea === "number") {
                                if (event.nativeEvent.pageX < this.obj.props.backTouchingArea) {
                                    return false;
                                }
                            }
                        }
                        return this.obj.props.gestureRecognizers.length > 0;
                    },
                    onResponderGrant: (event) => {
                        for (var i = 0; i < this.obj.props.gestureRecognizers.length; i++) {
                            var item = this.obj.props.gestureRecognizers[i];
                            item.touchesBegan(this.obj, event);
                        }
                    },
                    onResponderReject: (event) => {
                        for (var i = 0; i < this.obj.props.gestureRecognizers.length; i++) {
                            var item = this.obj.props.gestureRecognizers[i];
                            item.touchesCancelled(this.obj, event);
                        }
                    },
                    onResponderTerminationRequest: (event) => {
                        if (View.nextTouchResponder instanceof View) {
                            if (typeof View.nextTouchResponder.props.canCancelContentTouches === "boolean" && View.nextTouchResponder.props.canCancelContentTouches === true) {
                                return true;
                            }
                        }
                        return false;
                    },
                    onResponderTerminate: (event) => {
                        for (var i = 0; i < this.obj.props.gestureRecognizers.length; i++) {
                            var item = this.obj.props.gestureRecognizers[i];
                            item.touchesCancelled(this.obj, event);
                        }
                    },
                    onResponderMove: (event) => {
                        for (var i = 0; i < this.obj.props.gestureRecognizers.length; i++) {
                            var item = this.obj.props.gestureRecognizers[i];
                            item.touchesMoved(this.obj, event);
                        }
                    },
                    onResponderRelease: (event) => {
                        for (var i = 0; i < this.obj.props.gestureRecognizers.length; i++) {
                            var item = this.obj.props.gestureRecognizers[i];
                            item.touchesEnded(this.obj, event);
                        }
                    },
                };
            }
        }
        else {
            return {};
        }
    }
}
/**
 * View
 */
export class View extends React.Component {
    constructor(props) {
        super(props);
        this.origin = { x: 0, y: 0 };
        this.bounds = { width: 0, height: 0 };
        this.isFirstResponder = false;
        this.state = {};
        this.animateImp = new ViewAnimateImp(this);
        this.drawImp = new ViewDrawImp(this);
        this.layoutImp = new ViewLayoutImp(this);
        this.touchImp = new ViewTouchImp(this);
    }
    static setAnimate(stateBlock, duration = 250, aniType = "timing", animationConfig = undefined, completionBlock = undefined) {
        ViewAnimateImp.setAnimate(stateBlock, duration, aniType, animationConfig, completionBlock);
    }
    static setLayout(stateBlock, duration = 250, aniType = "timing", animationConfig = undefined, completionBlock = undefined) {
        ViewAnimateImp.setLayoutAnimate(stateBlock, duration, aniType, animationConfig, completionBlock);
    }
    static endEditing() {
        View.firstResponder && View.firstResponder.resignFirstResponder();
    }
    view() {
        return null;
    }
    becomeFirstResponder() { }
    resignFirstResponder() { }
    layoutSubviews() {
        this.forceUpdate();
    }
    customStyle() {
        return {};
    }
    customProps() {
        return {};
    }
    render() {
        let view = this.view();
        let children = this.props.children;
        if (this.props.animatable) {
            if (!!this.props.frame && (typeof this.props.frame.cx === "number" || typeof this.props.frame.cy === "number")) {
                return (React.createElement(RN.View, {style: this.layoutImp.wrapStyles(), pointerEvents: "box-none"}, React.createElement(RN.Animated.View, __assign({style: [this.drawImp.viewStyles(), this.layoutImp.viewStyles(), this.customStyle(), this.props.style]}, this.drawImp.viewProps(), this.layoutImp.viewProps(), this.touchImp.viewProps(), this.customProps()), view ? view : children)));
            }
            else {
                return (React.createElement(RN.Animated.View, __assign({style: [this.drawImp.viewStyles(), this.layoutImp.viewStyles(), this.customStyle(), this.props.style]}, this.drawImp.viewProps(), this.layoutImp.viewProps(), this.touchImp.viewProps(), this.customProps()), view ? view : children));
            }
        }
        else {
            if (!!this.props.frame && (typeof this.props.frame.cx === "number" || typeof this.props.frame.cy === "number")) {
                return (React.createElement(RN.View, {style: this.layoutImp.wrapStyles(), pointerEvents: "box-none"}, React.createElement(RN.View, __assign({style: [this.drawImp.viewStyles(), this.layoutImp.viewStyles(), this.customStyle(), this.props.style]}, this.drawImp.viewProps(), this.layoutImp.viewProps(), this.touchImp.viewProps(), this.customProps()), view ? view : children)));
            }
            else {
                return (React.createElement(RN.View, __assign({style: [this.drawImp.viewStyles(), this.layoutImp.viewStyles(), this.customStyle(), this.props.style]}, this.drawImp.viewProps(), this.layoutImp.viewProps(), this.touchImp.viewProps(), this.customProps()), view ? view : children));
            }
        }
    }
    componentDidMount() {
        this.animateImp.resetAnimationValues(this.props);
    }
    componentWillReceiveProps(newProps) {
        if (this.props.animatable) {
            this.animateImp.animateAnimationValues(newProps);
        }
        else {
            this.animateImp.resetAnimationValues(newProps);
        }
    }
    shouldComponentUpdate(newProps, newState) {
        for (var key in newProps) {
            if (newProps.hasOwnProperty(key) && this.props.hasOwnProperty(key)) {
                if (newProps[key] !== this.props[key]) {
                    return true;
                }
            }
            else {
                return true;
            }
        }
        for (var key in newState) {
            if (newState.hasOwnProperty(key) && this.state.hasOwnProperty(key)) {
                if (newState[key] !== this.state[key]) {
                    return true;
                }
            }
            else {
                return true;
            }
        }
        return false;
    }
}
View.defaultProps = {
    userInteractionEnabled: true,
    tag: 0,
    animatable: false,
    boundsRelative: false,
    clipsToBounds: false,
    backgroundColor: "#ffffff00",
    alpha: 1.0,
    hidden: false,
    tintColor: PresetColor.tintColor,
    tintColorDisabled: PresetColor.tintColorDisabled,
    cornerRadius: 0.0,
    borderColor: undefined,
    borderWidth: 0.0,
    lighted: false,
    reversed: false,
    gestureRecognizers: [],
    backTouchingArea: 44.0,
};
View.firstResponder = undefined;
View.currentTouchResponder = undefined;
View.nextTouchResponder = undefined;
