/// <reference path="../../typings/tsd.d.ts"/>
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { NavigationItem } from "./NavigationItem";
import { ActivityIndicatorView } from "./ActivityIndicatorView";
/**
 * ViewController
 */
export class ViewController extends React.Component {
    constructor(...args) {
        super(...args);
        this.lazyLoad = false;
        this.bounds = { width: 0, height: 0 };
        this._autoRelease = [];
        this._loaded = false;
        this.presented = false;
    }
    static navigationItem(navigationController, canGoBack) {
        let item = new NavigationItem();
        item.resetTitleView(this.title, navigationController);
        return item;
    }
    navigationController() {
        return this.props.navigationController;
    }
    loadView() {
        return React.createElement(RN.View, null);
    }
    loadingView() {
        return React.createElement(ActivityIndicatorView, {frame: { cx: 0, cy: 0 }, animating: true, tintColor: "#AAAAAA"});
    }
    viewDidLoad() { }
    viewWillUnload() { }
    viewWillAppear() { }
    viewDidAppear() { }
    viewWillDisappear() { }
    viewDidDisappear() { }
    viewWillLayoutSubviews() { }
    autoRelease(obj) {
        this._autoRelease.push(obj);
    }
    // Private
    componentDidMount() {
        this.viewDidLoad();
        RN.BackAndroid.addEventListener('hardwareBackPress', () => {
            if (!!this.props.presentingViewController) {
                this.dismissViewController();
                return true;
            }
            return false;
        });
        if (!!this.navigationController()) {
            this.navigationController().currentViewControllers.push(this);
            this.viewWillAppear();
            if (this.navigationController().currentViewControllers.length <= 1) {
                this.viewDidAppear();
            }
        }
        else if (!!this.props.presentingViewController) {
            this.viewWillAppear();
            setTimeout(() => {
                RN.InteractionManager.runAfterInteractions(() => {
                    this.viewDidAppear();
                });
            }, 200);
        }
        else {
            this.viewWillAppear();
            this.viewDidAppear();
        }
        if (this.lazyLoad) {
            RN.InteractionManager.runAfterInteractions(() => {
                this._loaded = true;
                this.forceUpdate();
            });
        }
    }
    componentWillUnmount() {
        this.viewWillUnload();
        if (!!this.navigationController()) {
            let newStack = [];
            for (let index = 0; index < this.navigationController().currentViewControllers.length; index++) {
                let element = this.navigationController().currentViewControllers[index];
                if (element !== this) {
                    newStack.push(element);
                }
            }
            this.navigationController().currentViewControllers = newStack;
        }
        else {
            this.viewWillDisappear();
            RN.InteractionManager.runAfterInteractions(() => {
                this.viewDidDisappear();
            });
        }
        this._autoRelease.forEach(obj => {
            if (typeof obj === "number") {
                clearTimeout(obj);
                clearInterval(obj);
            }
        });
    }
    presentViewController(ViewControllerClass, args, animated = true) {
        this.presentedViewController = React.createElement(ViewControllerClass, __assign({presentingViewController: this}, args));
        this.presented = false;
        this.viewWillDisappear();
        if (animated) {
            this.setState({}, () => {
                setTimeout(() => {
                    View.setAnimate(() => {
                        this.presented = true;
                        this.setState({});
                    }, 0, 'spring', { friction: 24, tension: 360, toValue: 0.0 }, () => {
                        this.viewDidDisappear();
                    });
                });
            });
        }
        else {
            this.presented = true;
            this.setState({});
            this.viewDidDisappear();
        }
    }
    dismissViewController(animated = true) {
        if (this.presentedViewController !== null && this.presentedViewController !== undefined) {
            this._dismissViewController(animated);
        }
        else if (this.props.presentingViewController !== null && this.props.presentingViewController !== undefined) {
            this.props.presentingViewController._dismissViewController(animated);
        }
    }
    _dismissViewController(animated = true) {
        this.viewWillAppear();
        if (animated) {
            this.presented = false;
            setTimeout(() => {
                View.setLayout(() => {
                    this.setState({});
                }, 500, 'spring', null, () => {
                    this.viewDidAppear();
                });
            });
            setTimeout(() => {
                this.presentedViewController = null;
                this.setState({});
            }, 500);
        }
        else {
            this.presented = false;
            this.presentedViewController = null;
            this.setState({});
            this.viewDidAppear();
        }
    }
    modal() {
        if (this.presentedViewController === null || this.presentedViewController === undefined) {
            return null;
        }
        return (React.createElement(View, {frame: { x: 0, y: 0, mx: 0, my: 0 }, margin: { top: !this.presented ? RN.Dimensions.get('window').height : 0.0 }, animatable: true}, this.presentedViewController));
    }
    render() {
        return (React.createElement(RN.View, {style: [{ marginTop: this.props.marginTop, marginBottom: this.props.marginBottom, flex: 1 }, this.props.style], onLayout: (event) => {
            if (this.bounds.width == event.nativeEvent.layout.width && this.bounds.height == event.nativeEvent.layout.height) {
                return;
            }
            this.bounds = { width: event.nativeEvent.layout.width, height: event.nativeEvent.layout.height };
            this.viewWillLayoutSubviews();
        }}, !this.lazyLoad || this._loaded ? this.loadView() : this.loadingView(), this.modal()));
    }
}
ViewController.title = "";
ViewController.defaultProps = {
    style: {
        backgroundColor: "white",
        flex: 1,
    }
};
